PROGRAM FRQCNT;

DEVICE = MEGA8;

{$I MEGA8_Register.INC}



CONST

  // TaktFrequenz in KHz
  TaktFrequenz : WORD = 12000;

  // bei 12,000 MHz -> 57.600
  UBRR_VAL : Byte = 12;




VAR
{
 B0..B2 -> 3 Bit Daten Low Eingang
 C0..C4 -> 5 Bit Daten High Eingang
 D0=RXD, D1=TXD -> UART
 D2 = TOR -> Torschaltung
 D3 = MR -> Zaehler Reset
 D4 frei
 D5 = CNT_IN -> Ausgang Biaerzaehler und Eingang uC Zaehler
 D6 = LED gruen
 D7 = LED rot (Torschaltung)
}

  TOR[PortD, 2] : bit;     // -> A (High-Aktiv)
  MR[PortD, 3] : bit;      // -> A (High-Aktiv)
  CNT_IN[PIND, 5] : bit;   // -> E

  LED_GN[PortD, 6] : bit;  // -> E (High-Aktiv)
  LED_RT[PortD, 7] : bit;  // -> E (High-Aktiv)


  Steuer_String : String[20];
  Anzeige_String : String[20];


  MW_01, MW_02, MW_03 : Byte;



{$I SYSTEM.INC}

{$I mega8_TC.INC}


// serielle Schnittstelle ohne Interrupt
{$I mega8_UART_oI.INC}



procedure InitPorts;
begin
  DDRB:=  %00000000;
  DDRC:=  %00000000;
  //       LLXCMTTR
  DDRD:=  %11001110;
  PortD:= %00000000;
end InitPorts;




procedure starte_Messung;
begin

  // Ruecksetzen der beiden externen Zaehler (74HCT193)
  SetBit(MR);
  ClearBit(MR);

  // Ruecksetzen des internen Zaehlers
  // im uC im Timer1
  TCNT1H:=0;
  TCNT1L:=0;

  // oeffnen der Torschaltung
  SetBit(LED_RT);
  SetBit(TOR);
  // Abwarten der Zaehlzeit
  wait_ms(100);
  // schliessen der Torschaltung
  ClearBit(TOR);
  ClearBit(LED_RT);

end starte_Messung;







procedure lese_Messwert;
begin

  // lesen der Messwerte aus den beiden externen Zaehlern (74HCT193)
  MW_02:=PINB;
  MW_03:=PINC;

  // Packen der 3 Bit aus PortB und der
  // 5 Bit aus PortC in ein Bytewert
  MW_01:=MW_02 and %00000111;
  SHL(MW_03);
  SHL(MW_03);
  SHL(MW_03);
  MW_01:=MW_01 or MW_03;

  // lesen des 16 Bit Zaehlers
  MW_02:=TCNT1L;
  MW_03:=TCNT1H;

end lese_Messwert;









procedure uebertrage_Messwert;
begin

  // Packen der Sendestring im Format
  // M030201+CR+LF
  ByteToString(MW_03);
  Anzeige_String:='M'+Write_String;
  ByteToString(MW_02);
  Anzeige_String:=Anzeige_String+Write_String;
  ByteToString(MW_01);
  Write_String:=Anzeige_String+Write_String;

  // Senden von Write_String+CR+LF
  WriteLn;

end uebertrage_Messwert;





procedure Steuer_String_auswerten;
var
  Buchstabe : Char;
  Zeichen_1 : Char;
  Zeichen_2 : Char;
  Laenge : Byte;
begin
  Laenge:=Length(Steuer_String);

  if Laenge>0 then
    Buchstabe:=Steuer_String[1];
    case Buchstabe of

            // S
      'S' : if Laenge=1 then
              starte_Messung;
              lese_Messwert;
              uebertrage_Messwert;
            else
              Write_String:='x01';
              WriteLn;
            endif;

      |

    else
      Write_String:='x02';
      WriteLn;
    endcase;

  endif; { if Laenge>0 then }

end Steuer_String_auswerten;








procedure Steuer_String_lesen;
var
  akt_Zeichen : Char;
begin
  if Zeichen<>$0D then // 0D wird ignoriert

    if Zeichen=$0A then  // bei 0A wird ausgerwertet
      Steuer_String_auswerten;
      Steuer_String:='';
    else
      akt_Zeichen:=ByteToChar(Zeichen);
      Steuer_String:= Steuer_String + akt_Zeichen; // sonst nur abgespeichert
    endif; { if akt_Zeichen<>char($0A) }

  endif; { if akt_Zeichen<>char($0D) then }

end Steuer_String_lesen;
















BEGIN

  // Initialisierung der Ports
  InitPorts;

  // initialisierung der seriellen Schnittstelle
  INIT_SER_PORT;

  // Grundeinstelung der Ports
  ClearBit(TOR);
  ClearBit(MR);

  ClearBit(LED_GN);
  ClearBit(LED_RT);

  // Ruecksetzen der beiden externen Zaehler (74HCT193)
  SetBit(MR);
  ClearBit(MR);

  // Initialisierung und Ruecksetzen des internen Zaehlers
  // im uC im Timer1 (kein Interrupt)
  Init_TIMER1_external_Counter;


//  sende_Zeichen($40);

  loop

    RX_Puffer_abfragen;

    if Zeichen>0 then
      Steuer_String_lesen;
    endif;



  endloop;



END FRQCNT.
