PROGRAM Z80HWE;

DEVICE = MEGA1284P;

{$I mega1284_Register.INC}



CONST

  // ProcClock = 18432000;       {Hertz}
  //  UBRR_VAL : Byte = 79;  // 14400 Baud
  //  UBRR_VAL : Byte = 59;  // 19200 Baud
  //  UBRR_VAL : Byte = 29;  // 38400 Baud
  //  UBRR_VAL : Byte = 19;  // 57600 Baud

  //  UBRR_VAL : Byte = 9;  // 115200 Baud
    UBRR_VAL : Byte = 4;  // 230400 Baud
  //  UBRR_VAL : Byte = 2;  // 384000 Baud



  XON  : Byte = $11;
  XOFF : Byte = $13;


  // Befehlscode fuer IORQ
  IORQ_LED_Ein    : Byte = 01;
  IORQ_LED_Aus    : Byte = 02;
  IORQ_CONOUT     : Byte = 03;
  IORQ_CONIN      : Byte = 04;
  IORQ_CONST      : Byte = 05;
  IORQ_START_PROG : Byte = 06;
  IORQ_RAM2ARR    : Byte = 07;
  IORQ_ARR2RAM    : Byte = 08;
  IORQ_STD2UC     : Byte = 09;
  IORQ_RD_PC2ARR  : Byte = 10;
  IORQ_BDOS2RAM   : Byte = 11;
  IORQ_CCP2RAM    : Byte = 12;
  IORQ_WR_ARR2PC  : Byte = 14;
  IORQ_DISK2RAM   : Byte = 15;
  IORQ_LIST 	  : Byte = 16;
  IORQ_TIME 	  : Byte = 17;



VAR

  // Globale Variablen

{
PA0 - A16_RAM   - A
PA1 - A17_RAM   - A
PA2 - A18_RAM   - A
PA3 - RESET_Z80 - A
PA4 - CLK_Z80   - A
PA5 - OE_RAM    - A
PA6 - WE_RAM    - A
PA7 - G_RAMSW   - A
}
  A16_RAM[PortA,0]   : bit; // A
  A17_RAM[PortA,1]   : bit; // A
  A18_RAM[PortA,2]   : bit; // A
  RESET_Z80[PortA,3] : bit; // A
  CLK_Z80[PortA,4]   : bit; // A
  OE_RAM_A[PortA,5]  : bit; // E/A
  WE_RAM_A[PortA,6]  : bit; // E/A
  G_RAMSW[PortA,7]   : bit; // A


  OE_RAM_D[DDRA,5]    : bit; // A
  WE_RAM_D[DDRA,6]    : bit; // A


{
PB0 - M1_Z80    - E
PB1 - MREQ_Z80  - E
PB2 - IORQ_Z80  - E (INT2)
PB3 - RD_Z80    - E
PB4 - WR_Z80    - E
PB5 - MOSI      - E
PB6 - MISO      - E
PB7 - SCK       - E
}
  M1_Z80_E[PinB,0]    : bit; // E
  MREQ_Z80_E[PinB,1]  : bit; // E
  IORQ_Z80_E[PinB,2]  : bit; // E
  RD_Z80_E[PinB,3]    : bit; // E
  WR_Z80_E[PinB,4]    : bit; // E




{
PD0 - RXD0      - E
PD1 - TXD0      - A
PD2 - RXD1      - E (bei ATMEGA 1284P)
PD3 - TXD1      - A (bei ATMEGA 1284P)
PD4 - frei      - E
PD5 - frei      - E
PD6 - frei      - E
PD7 - LED_BL    - A
}
  LED_BL[PortD,7]    : bit; // E

  Array_Pos : Word;

  ConsoleIn : Byte;

  Steuer_String : String[50];
  Anzeige_String : String[80];

  // Sektor-Puffer
  Sektor_Puffer : Array [0..127] of Byte;
  Sektor_Puffer_Zeiger : Word;

  // STD
  SECTOR, TRACK_L, TRACK_H, DISKNO : Byte;

  Datum_gesetzt : Boolean;



CONST

// BIOS - Z80-Code
{$I CPD6_TXT.INC}


// BDOS - Z80-Code
{$I BDOS_TXT.INC}


// CCP - Z80-Code
{$I CCP_TXT.INC}


// RAM-Floppy
{$I DISK_TXT.INC}


{$I SYSTEM.INC}

{$I mega16_128_UART0_oI.INC}

{$I mega16_128_UART1_oI_XON.INC}

// *** INITIALISIERUNG ***


procedure InitPorts;
begin

{                DIR  Pegel
PA0 - A16_RAM   - A     0
PA1 - A17_RAM   - A     0
PA2 - A18_RAM   - A     0
PA3 - RESET_Z80 - A     1
PA4 - CLK_Z80   - A     0
PA5 - OE_RAM    - E     1
PA6 - WE_RAM    - E     1
PA7 - G_RAMSW   - A     0 - Durchschalten der Z80 Steuerleitungen
}
  DDRA  :=  %10011111;
  PortA :=  %01101000;

{
PB0 - M1_Z80    - E
PB1 - MREQ_Z80  - E
PB2 - IORQ_Z80  - E (INT2)
PB3 - RD_Z80    - E
PB4 - WR_Z80    - E
PB5 - MOSI      - E
PB6 - MISO      - E
PB7 - SCK       - E
}
  DDRB  :=  %00000000;
  PortB :=  %11111111;

{
PC0..PC7 = Daten (8)
}
  DDRC  :=  %00000000;
  PortC :=  %00000000;

{
PD0 - RXD0      - E
PD1 - TXD0      - A
PD2 - RXD1      - E (bei ATMEGA 1284P)
PD3 - TXD1      - A (bei ATMEGA 1284P)
PD4 - frei      - E
PD5 - frei      - E
PD6 - frei      - E
PD7 - LED_BL    - A
}
  DDRD  :=  %10001010;
  PortD :=  %00000000;


end InitPorts;




// *** Z80 Hardware-Steuerprogramme ***



procedure sende_10_Takte;
var
  Zaehler : Byte;
begin

  for Zaehler:=1 to 100 do
    SetBit(CLK_Z80);
    NOP;
    NOP;
    ClearBit(CLK_Z80);
  endfor;

end sende_10_Takte;




procedure warte_solange_RD_High;
begin

  // warten bis /RD aktiv wird (Low)
  while BitHigh(RD_Z80_E) do

    if BitHigh(RD_Z80_E) then
      SetBit(CLK_Z80);
    endif;

     if BitHigh(RD_Z80_E) then
      ClearBit(CLK_Z80);
    endif;

  endwhile; // while BitHigh(RD_Z80_E) do

end warte_solange_RD_High;





procedure warte_solange_RD_Low;
begin

  // warten bis /RD inaktiv wird (High)
    while BitLow(RD_Z80_E) do
      SetBit(CLK_Z80);
      NOP;
      ClearBit(CLK_Z80);
    endwhile; // while BitLow(RD_Z80_E) do

end warte_solange_RD_Low;




procedure warte_solange_IORQ_High;
begin

  // warten bis /IORQ aktiv wird (Low)
  while BitHigh(IORQ_Z80_E) do

    if BitHigh(IORQ_Z80_E) then
      SetBit(CLK_Z80);
    endif;

     if BitHigh(IORQ_Z80_E) then
      ClearBit(CLK_Z80);
    endif;

  endwhile; // while BitLow(IORQ_Z80_E) do

end warte_solange_IORQ_High;




procedure warte_solange_IORQ_Low;
begin

  // warten bis /IORQ inaktiv wird (High)
  while BitLow(IORQ_Z80_E) do
    SetBit(CLK_Z80);
    NOP;
    ClearBit(CLK_Z80);
  endwhile; // while BitLow(IORQ_Z80_E) do

end warte_solange_IORQ_Low;




procedure lade_ARRAY_in_RAM;
begin


  // 3. Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  for Array_Pos:=0 to Laenge_CPD6_Array do

    warte_solange_RD_High;

    // 5. Einschreiben der Daten vom CPD5_ARR in den RAM
    // an der aktuellen Adresse (entspricht Array_Pos)

    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    // Anlegen der Z80-Daten
    PORTC := CPD6_ARR[Array_Pos];

    NOP;
    NOP;
    NOP;

    ClearBit(WE_RAM_A);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    SetBit(WE_RAM_A);

    // 6. Anlegen von NOP an den Datenport fuer die Z80
    PORTC := 0;  // NOP

    warte_solange_RD_Low;

    // Daten-Port auf Lesen
    DDRC  :=  %00000000;

  // 8. Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for Array_Pos:=0 to Laenge_CPD6_Array do }


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);


  // 11. Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);



end lade_ARRAY_in_RAM;














// laedt das Z80-BIOS in den RAM
// und startet es durch RESET
procedure Z80_RESET_und_URLOAD;
begin

  // 0. /RESET auf High setzen
  SetBit(RESET_Z80);

  sende_10_Takte;

  // 1. /RESET auf LOW setzen
  ClearBit(RESET_Z80);

  // 2a. Senden von min. 3 Takten (10 Takte)
  sende_10_Takte;

  // 2b. /RESET auf High setzen
  SetBit(RESET_Z80);

  lade_ARRAY_in_RAM;

  // 9. /RESET auf LOW setzen
  ClearBit(RESET_Z80);

  // 10a. Senden von min. 3 Takten (10 Takte)
  sende_10_Takte;


  // 10b. /RESET auf High setzen
  SetBit(RESET_Z80);

end Z80_RESET_und_URLOAD;








procedure IORQ_Ende;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;

end IORQ_Ende;








procedure Z80_CONOUT;
var
  IO_Wert : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist das auszugebende Zeichen
  warte_solange_IORQ_High;

  IO_Wert := PinC;

  ByteToString(IO_Wert);
  Write_String:='o'+Write_String;
//  WriteLn_SER1_XON;
  WriteLn_SER0;

  IORQ_Ende;

end Z80_CONOUT;








procedure Z80_LIST;
var
  IO_Wert : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist das auszugebende Zeichen
  warte_solange_IORQ_High;

  IO_Wert := PinC;

  ByteToString(IO_Wert);
  Write_String:='l'+Write_String;
  WriteLn_SER0;

  IORQ_Ende;

end Z80_LIST;









procedure Z80_CONIN;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=ConsoleIn;

  IORQ_Ende;

end Z80_CONIN;







procedure CONSOLE_CONOUT(Byte_Wert : Byte);
begin
  ByteToString(Byte_Wert);
  Write_String:='o'+Write_String;

  WriteLn_SER0;
//  WriteLn_SER1_XON;

end CONSOLE_CONOUT;






procedure CONSOLE_CONOUT_Anzeige_String;
var
  c1 : Char;
  Laenge : Byte;
  b1 : Byte;
  b2 : Byte;
begin

  Laenge:=Length(Anzeige_String);
  for b1:=1 to Laenge do
    c1:=Anzeige_String[b1];
    b2:=CharToByte(c1);
    CONSOLE_CONOUT(b2);
  endfor;

  CONSOLE_CONOUT($0D);
  CONSOLE_CONOUT($0A);

  Anzeige_String:='';

end CONSOLE_CONOUT_Anzeige_String;





// Start eines beliebigen Testprogramms des uC
// durch die Z80
procedure START_PROG;
var
  b1 : Byte;
  b2 : Byte;
  b3 : Byte;
  Adresse : Word;
begin
  Anzeige_String:='';
  Adresse:=0;

{
  b3:=5;
  for b1:=1 to 8 do
    for b2:=1 to 16 do
      Sektor_Puffer[Adresse]:=b3;
      Inc(Adresse);
      Inc(b3);
    endfor;
  endfor;

  Adresse:=0;
}


  for b1:=1 to 8 do
    for b2:=1 to 16 do
      b3:=Sektor_Puffer[Adresse];
      Inc(Adresse);
      ByteToString(b3);
      Anzeige_String:=Anzeige_String+Write_String;
      Anzeige_String:=Anzeige_String+' ';
    endfor;
    CONSOLE_CONOUT_Anzeige_String;
  endfor;

  CONSOLE_CONOUT($0D);
  CONSOLE_CONOUT($0A);

  IORQ_Ende;

end START_PROG;









// Kopieren des RAMs ab Adresse in den Sektor_Puffer
procedure START_RAM2ARR;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
  RAM_PORT : Byte;
begin


  // warten bis /IORQ inaktiv wird (High)


// warte_solange_IORQ_Low
ASM;

M_ASM_warte_solange_IORQ_Low_R2A1:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_Low_R2A1

ENDASM;


  // Das naechste Byte ist das auszugebende Zeichen

// warte_solange_IORQ_High
ASM;

M_ASM_warte_solange_IORQ_High_R2A2:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_High_A_R2A2

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_High_R2A2


M_ASM_warte_solange_IORQ_High_A_R2A2:


ENDASM;



  RAM_ADR := PinC;



// warte_solange_IORQ_Low
ASM;

M_ASM_warte_solange_IORQ_Low_R2A3:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_Low_R2A3

ENDASM;



  // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl



// warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_R2A4:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A_R2A4

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_R2A4

M_ASM_warte_solange_RD_High_A_R2A4:

ENDASM;



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_R2A5:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_Low_R2A5

ENDASM;



  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird der RAM gelesen ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := RAM_ADR AND %00000111;
  RAM_PORT := PortA AND %11111000;
  RAM_ADR := RAM_ADR OR RAM_PORT;
  PortA := RAM_ADR;


  for SP_Zaehler:=0 to 127 do


 // warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_R2A6:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A_R2A6

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_R2A6

M_ASM_warte_solange_RD_High_A_R2A6:

ENDASM;



    // Daten-Port auf Lesen
    DDRC  :=  %00000000;

    ClearBit(OE_RAM_A);

    NOP;
    NOP;
    NOP;

    RAM_Wert := PinC;
    Sektor_Puffer[SP_Zaehler]:=RAM_Wert;

    SetBit(OE_RAM_A);

    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;


    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_R2A7:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_Low_R2A7

ENDASM;



  // 8. Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for Array_Pos:=0 to Laenge_CPD6_Array do }


  RAM_ADR := PortA AND %11111000;
  PortA := RAM_ADR;



  // *** Einschreiben von RET ($C9) an Z80




// warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_R2A8:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A_R2A8

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_R2A8

M_ASM_warte_solange_RD_High_A_R2A8:

ENDASM;



  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_R2A9:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_Low_R2A9

ENDASM;


  // Daten-Port auf Lesen
  DDRC  :=  %00000000;

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end START_RAM2ARR;








// Kopieren des Sektor_Puffer in den RAMs ab DMA-Adresse
procedure START_ARR2RAM;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
  RAM_PORT : Byte;
begin



  // warten bis /IORQ inaktiv wird (High)
// warte_solange_IORQ_Low
ASM;

M_ASM_warte_solange_IORQ_Low_A2R1:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_Low_A2R1

ENDASM;



  // Das naechste Byte ist das auszugebende Zeichen
// warte_solange_IORQ_High
ASM;

M_ASM_warte_solange_IORQ_High_A2R2:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_High_A_A2R2

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_High_A2R2

M_ASM_warte_solange_IORQ_High_A_A2R2:

ENDASM;



  RAM_ADR := PinC;



// warte_solange_IORQ_Low
ASM;

M_ASM_warte_solange_IORQ_Low_A2R3:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_Low_A2R3

ENDASM;



  // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl

// warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_A2R4:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A_A2R4

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A2R4

M_ASM_warte_solange_RD_High_A_A2R4:

ENDASM;



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_A2R5:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_Low_A2R5

ENDASM;


  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird der RAM gelesen ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := RAM_ADR AND %00000111;
  RAM_PORT := PortA AND %11111000;
  RAM_ADR := RAM_ADR OR RAM_PORT;
  PortA := RAM_ADR;


  for SP_Zaehler:=0 to 127 do




 // warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_A2R6:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A_A2R6

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A2R6


M_ASM_warte_solange_RD_High_A_A2R6:


ENDASM;


    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    RAM_Wert := Sektor_Puffer[SP_Zaehler];
    PortC := RAM_Wert;

    ClearBit(WE_RAM_A);

    NOP;
    NOP;
    NOP;

    SetBit(WE_RAM_A);

    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_A2R7:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_Low_A2R7

ENDASM;



  // Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for Array_Pos:=0 to Laenge_CPD6_Array do }


  RAM_ADR := PortA AND %11111000;
  PortA := RAM_ADR;


  // *** Einschreiben von RET ($C9) an Z80



// warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_A2R8:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A_A2R8

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 3
        RJMP       M_ASM_warte_solange_RD_High_A2R8

M_ASM_warte_solange_RD_High_A_A2R8:

ENDASM;


  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_A2R9:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 3
        RJMP       M_ASM_warte_solange_RD_Low_A2R9

ENDASM;


  // Daten-Port auf Lesen
  DDRC  :=  %00000000;

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end START_ARR2RAM;









// Kopieren des BDOS in den RAM
procedure START_BDOS2RAM;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

 // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl

  warte_solange_RD_High;
  warte_solange_RD_Low;

  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird in den RAM geschrieben ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := PortA AND %11111000;
  PortA := RAM_ADR;


  for SP_Zaehler:=0 to Laenge_BDOS_Array do

    warte_solange_RD_High;


    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    RAM_Wert := BDOS_ARR[SP_Zaehler];
    PortC := RAM_Wert;

    ClearBit(WE_RAM_A);

    NOP;
    NOP;
    NOP;

    SetBit(WE_RAM_A);


    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP

    warte_solange_RD_Low;

  // Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for SP_Zaehler:=0 to Laenge_BDOS_Array do }



  // *** Einschreiben von RET ($C9) an Z80

  warte_solange_RD_High;

  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET

  warte_solange_RD_Low;

  // Daten-Port auf Lesen
  DDRC  :=  %00000000;

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end START_BDOS2RAM;






// Kopieren des CCP in den RAM
procedure START_CCP2RAM;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;


  // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl

  warte_solange_RD_High;
  warte_solange_RD_Low;


  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird in den RAM geschrieben ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := PortA AND %11111000;
  PortA := RAM_ADR;


  for SP_Zaehler:=0 to Laenge_CCP_Array do

    warte_solange_RD_High;


    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    RAM_Wert := CCP_ARR[SP_Zaehler];
    PortC := RAM_Wert;

    ClearBit(WE_RAM_A);

    NOP;
    NOP;
    NOP;

    SetBit(WE_RAM_A);


    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP

    warte_solange_RD_Low;

  // Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for SP_Zaehler:=0 to Laenge_BDOS_Array do }



  // *** Einschreiben von RET ($C9) an Z80

  warte_solange_RD_High;

  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET

  warte_solange_RD_Low;

  // Daten-Port auf Lesen
  DDRC  :=  %00000000;

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end START_CCP2RAM;






// Kopieren des Disk-Images in den RAM
procedure COPY_DISK_TO_RAM;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;


  // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl

  warte_solange_RD_High;
  warte_solange_RD_Low;


  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird in den RAM geschrieben ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := PortA AND %11111000;
  PortA := RAM_ADR OR 00000001;


  for SP_Zaehler:=0 to Laenge_DISK_Array do

    warte_solange_RD_High;

    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    RAM_Wert := DISK_ARR[SP_Zaehler];
    PortC := RAM_Wert;

    ClearBit(WE_RAM_A);

    NOP;
    NOP;
    NOP;

    SetBit(WE_RAM_A);

    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP

    warte_solange_RD_Low;

  // Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for SP_Zaehler:=0 to Laenge_BDOS_Array do }



  // *** Einschreiben von RET ($C9) an Z80

  warte_solange_RD_High;

  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET

  warte_solange_RD_Low;

  // Daten-Port auf Lesen
  DDRC  :=  %00000000;




  RAM_ADR := PortA AND %11111000;
  PortA := RAM_ADR;



  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end COPY_DISK_TO_RAM;


















procedure STD_an_uC;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist der SECTOR
  warte_solange_IORQ_High;

  SECTOR := PinC;


  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist der TRACK_L
  warte_solange_IORQ_High;

  TRACK_L := PinC;


  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist der TRACK_H
  warte_solange_IORQ_High;

  TRACK_H := PinC;


  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist der DISKNO
  warte_solange_IORQ_High;

  DISKNO := PinC;


  IORQ_Ende;


end STD_an_uC;






procedure STD_an_PC_senden;
begin

  // Sector, Track, Drive (STD) = s SE TH TL DR
  ByteToString(SECTOR);
  Anzeige_String:='s'+Write_String;
  ByteToString(TRACK_H);
  Anzeige_String:=Anzeige_String+Write_String;
  ByteToString(TRACK_L);
  Anzeige_String:=Anzeige_String+Write_String;
  ByteToString(DISKNO);
  Write_String:=Anzeige_String+Write_String;

  // Senden der STD
  WriteLn_SER0;

end STD_an_PC_senden;







// liest die Bytes aus der SteuerString in den Sektor_Puffer
procedure Steuer_String_To_Sector;
var
  b1 : Byte;
  b2 : Byte;
  b3 : Byte;
  c1 : Char;
begin
  b2:=2;
  for b1:=0 to 15 do
    c1:=Steuer_String[b2];
    Inc(b2);
    Write_String:=c1;
    c1:=Steuer_String[b2];
    Inc(b2);
    Write_String:=Write_String+c1;
    b3:=StringToByte;
    Sektor_Puffer[Sektor_Puffer_Zeiger]:=b3;
    Inc(Sektor_Puffer_Zeiger);
  endfor;

end Steuer_String_To_Sector;







// liest die Bytes aus der SteuerString in den Sektor_Puffer aber komprimiert
procedure Steuer_String_To_Sector_Komp;
var
 // x1 : Byte;
  x2 : Byte;
  DB7 : Byte;
  DB8 : Byte;
  BitPos_DB8 : Byte;
  BitPos_DB7 : Byte;
  c1 : Char;
  Pos_SS : Byte;
  ein_Bit : Boolean;

begin
  BitPos_DB8:=7;
  DB8:=$00;
  for Pos_SS:=2 to 38 do

    c1:=Steuer_String[Pos_SS];
    DB7:=CharToByte(c1);

    BitPos_DB7:=7;
    for x2:=0 to 6 do

      Dec(BitPos_DB7); // 6..0
      ein_Bit:=Read_Bit_in_Byte(DB7,BitPos_DB7);

      if ein_Bit then
        DB8:=Set_Bit_in_Byte(DB8,BitPos_DB8);
      else
        DB8:=Clear_Bit_in_Byte(DB8,BitPos_DB8);
      endif;

      if BitPos_DB8=0 then

        Sektor_Puffer[Sektor_Puffer_Zeiger]:=DB8;
        Inc(Sektor_Puffer_Zeiger);

        BitPos_DB8:=7;
        DB8:=$00;

      else
        Dec(BitPos_DB8);
      endif;

    endfor; { for x2:=0 to 6 do }

  endfor; { for x1:=1 to 18 do }

end Steuer_String_To_Sector_Komp;













// liest die Bytes aus der SteuerString in den DATE und TIME-Puffer
procedure Steuer_String_To_TIME;
var
  b1 : Byte;
  b2 : Byte;
  c1 : Char;
begin


  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;


  // Date Low-Teil
  c1:=Steuer_String[4];
  Write_String:=c1;
  c1:=Steuer_String[5];
  Write_String:=Write_String+c1;
  b1:=StringToByte;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  // Date High-Teil
  c1:=Steuer_String[2];
  Write_String:=c1;
  c1:=Steuer_String[3];
  Write_String:=Write_String+c1;
  b1:=StringToByte;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  // HOUR
  c1:=Steuer_String[6];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[7];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  // MIN
  c1:=Steuer_String[8];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[9];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  // SEC
  c1:=Steuer_String[10];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[11];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  Datum_gesetzt:=true;

end Steuer_String_To_TIME;












procedure Steuer_String_auswerten;
var
  Buchstabe : Char;
  Zeichen_1 : Char;
  Zeichen_2 : Char;
  Laenge : Byte;
begin
  Laenge:=Length(Steuer_String);

  if Laenge>0 then
    Buchstabe:=Steuer_String[1];
    case Buchstabe of

            // T5B 2+1 Senden eines Zeichens an CONIN des Mikrocontrollers
      'T' : if Laenge=3 then
              Zeichen_1:=Steuer_String[2];
              Zeichen_2:=Steuer_String[3];
              Write_String:=Zeichen_1+Zeichen_2;
              ConsoleIn:=StringToByte;
            else
              Write_String:='x01';
              WriteLn_SER0;
            endif;

      |

            // UD0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF  16*2+1
      'U' : if Laenge=33 then // ein Sektor-Abschnitt
              Steuer_String_To_Sector;
            else
              Write_String:='x03';
              WriteLn_SER0;
            endif;

      |

          // RB01B02B03B04..B36 36+1
      'R' : if Laenge=38 then // ein Sektor-Abschnitt
              Steuer_String_To_Sector_Komp;
            else
              Write_String:='x07';
              WriteLn_SER0;
            endif;
      |


          // D329B133357 = D + Datum(4) + Zeit(6)
      'D' : if Laenge=11 then
              Steuer_String_To_TIME;
            else
              Write_String:='x09';
              WriteLn_SER0;
            endif;
      |


    else
      Write_String:='x02';
      WriteLn_SER0;
    endcase;

  endif; { if Laenge>0 then }



end Steuer_String_auswerten;




procedure Steuer_String_lesen;
var
  akt_Zeichen : Char;
  empf_Zeichen : Byte;
begin
  empf_Zeichen:=UDR0;

  if empf_Zeichen<>$0D then // 0D wird ignoriert

    if empf_Zeichen=$0A then  // bei 0A wird ausgerwertet
      Steuer_String_auswerten;
      Steuer_String:='';
    else
      akt_Zeichen:=ByteToChar(empf_Zeichen);
      Steuer_String:= Steuer_String + akt_Zeichen; // sonst nur abgespeichert
    endif; { if akt_Zeichen<>char($0A) }

  endif; { if akt_Zeichen<>char($0D) then }


end Steuer_String_lesen;






procedure XON_lesen;
var
  empf_Zeichen : Byte;
begin
  empf_Zeichen:=UDR1;

  if empf_Zeichen=XON then

    SER1_warten:=false;

  endif; { if empf_Zeichen=XON then }

end XON_lesen;







procedure UP_SET_TIME;
var
  LeseSchleife : Word;
begin

  Write_String:='d';
  WriteLn_SER0;

  Datum_gesetzt:=false;

  LeseSchleife:=60000;

    while LeseSchleife>5 do

      if BitHigh(RXC0) then

        Steuer_String_lesen;

        if Datum_gesetzt then
          LeseSchleife:=2;  // Ende
        else
          LeseSchleife:=60000;
        endif; { if Datum_gesetzt then }

      endif;

      Dec(LeseSchleife);
    endwhile;

  if LeseSchleife>2 then
    Write_String:='x10';
    WriteLn_SER0;
  endif;

end UP_SET_TIME;








procedure Sektor_von_PC_lesen;
var
  b1 : Byte;
  b2 : Byte;
  SPZ : Word;
  LeseSchleife : Word;
begin

  Sektor_Puffer_Zeiger:=0;
  SPZ:=0;

  for b1:=0 to 7 do
    ByteToString(b1);
    Write_String:='u'+Write_String;
    WriteLn_SER0;

    LeseSchleife:=60000;
    while LeseSchleife>5 do

      if BitHigh(RXC0) then
        Steuer_String_lesen;
        if SPZ=Sektor_Puffer_Zeiger then
          LeseSchleife:=60000;
        else
          LeseSchleife:=1;  // Ende
          SPZ:=Sektor_Puffer_Zeiger;
        endif;
      endif;

      Dec(LeseSchleife);
    endwhile;

    if LeseSchleife>0 then
      Write_String:='x04';
      WriteLn_SER0;
    endif;

  endfor;

end Sektor_von_PC_lesen;






procedure Sektor_von_PC_lesen_Komp;
var
  b1 : Byte;
  b2 : Byte;
  SPZ : Word;
  LeseSchleife : Word;
begin

  Sektor_Puffer_Zeiger:=0;
  SPZ:=0;

  for b1:=0 to 3 do
    ByteToString(b1);
    Write_String:='r'+Write_String;
    WriteLn_SER0;

    LeseSchleife:=60000;
    while LeseSchleife>5 do

      if BitHigh(RXC0) then
        Steuer_String_lesen;
        if SPZ=Sektor_Puffer_Zeiger then
          LeseSchleife:=60000;
        else
          LeseSchleife:=1;  // Ende
          SPZ:=Sektor_Puffer_Zeiger;
        endif;
      endif;

      Dec(LeseSchleife);
    endwhile;

    if LeseSchleife>0 then
      Write_String:='x08';
      WriteLn_SER0;
    endif;

  endfor;

end Sektor_von_PC_lesen_Komp;










procedure Sektor_an_PC_senden;
var
  b1 : Byte;
  b2 : Byte;
  b3 : Byte;
begin

  Sektor_Puffer_Zeiger:=0;

  for b1:=0 to 7 do

    ByteToString(b1);
    Anzeige_String:='v'+Write_String;

    for b2:=0 to 15 do
      b3:=Sektor_Puffer[Sektor_Puffer_Zeiger];
      Inc(Sektor_Puffer_Zeiger);
      ByteToString(b3);
      Anzeige_String:=Anzeige_String+Write_String;
    endfor; { for b2:=0 to 15 do }

    Write_String:=Anzeige_String;
    WriteLn_SER0;

  endfor; { for b1:=0 to 7 do }

end Sektor_an_PC_senden;








procedure Auswerten_IORQ;
var
  IO_Wert : Byte;
begin

  if BitLow(WR_Z80_E) then

    // Daten-Port auf Lesen
    DDRC  :=  %00000000;

    IO_Wert := PinC;

    case IO_Wert of

      IORQ_LED_Ein     : SetBit(LED_BL);
                         IORQ_Ende;
      |

      IORQ_LED_Aus     : ClearBit(LED_BL);
                         IORQ_Ende;
      |

      IORQ_CONOUT      : Z80_CONOUT;
      |

      IORQ_CONIN       : Z80_CONIN;
                         ConsoleIn:=0;
      |

      IORQ_CONST       : Z80_CONIN;
      |

      IORQ_START_PROG  : START_PROG;
      |

      IORQ_RAM2ARR     : START_RAM2ARR;
      |

      IORQ_ARR2RAM     : START_ARR2RAM;
      |

      IORQ_STD2UC      : STD_an_uC;
      |

      IORQ_RD_PC2ARR   : STD_an_PC_senden;
                         Sektor_von_PC_lesen_Komp;
      |

      IORQ_BDOS2RAM    : START_BDOS2RAM;
      |

      IORQ_CCP2RAM     : START_CCP2RAM;
      |

      IORQ_WR_ARR2PC   : STD_an_PC_senden;
                         Sektor_an_PC_senden;
      |

      IORQ_DISK2RAM    : COPY_DISK_TO_RAM;
      |

      IORQ_LIST        : Z80_LIST;
      |

      IORQ_TIME        : UP_SET_TIME;
      |



    endcase; { case IO_Wert of }

  endif; // if BitLow(WR_Z80) then

end Auswerten_IORQ;






BEGIN


  InitPorts;
  INIT_SER0_PORT;
  INIT_SER1_PORT;

  ConsoleIn:=0;
  Steuer_String:='';
  Datum_gesetzt:=false;

  sende_Zeichen0(64);  // @  = Ruecksetzen der Hardware


  // kurzes Blinken zum Start
  SetBit(LED_BL);
  wait_ms(100);
  ClearBit(LED_BL);
  wait_ms(100);

  // Laden des Z80-Bios
  Z80_RESET_und_URLOAD;


  // Endlosschleife fuer Takterzeugung
  // Taktfrequenz etwa 2 MHz
  LOOP

    SetBit(CLK_Z80);      // 2 Takte

ASM;

    LDS      R16, UCSR0A
    SBRC     R16, 7
    CALL     Steuer_String_lesen;

ENDASM;

    ClearBit(CLK_Z80);    // 2 Takte

ASM;

    SBIS     PinB, 2
    CALL     AUSWERTEN_IORQ

ENDASM;




    SetBit(CLK_Z80);      // 2 Takte

ASM;

    LDS      R16, UCSR1A
    SBRC     R16, 7
    CALL     XON_lesen;

ENDASM;

    ClearBit(CLK_Z80);    // 2 Takte

ASM;

    SBIS     PinB, 2
    CALL     AUSWERTEN_IORQ

ENDASM;





    // LDS  = 1 Takt
    // SBRC = 1 Takt
    // SBIS = 1 Takt
    // RJMP = 2 Takte

    // 18,4 / 9 = 2,1 MHz

  ENDLOOP;                // 2 Takte


END Z80HWE.
