PROGRAM VGAGEN2P;

DEVICE = MEGA644P;

{$I MEGA644P_Register.INC}

{$I mega644P_TC.INC}


{
F_CPU = 22000000
F_KHz_CPU = F_CPU/1000 ; CPU Frequenz in KHz
CTC_WERT = 31750*22/1000
}

CONST
  CTC_WERT : Word = 698;

  // VERTICAL LINE CONTROL
  VL_RESET    : Byte = 00;
  VL_VER_IMP  : Byte = 01;
  VL_OUT_VA   : Byte = 02;
  VL_INC_VA   : Byte = 03;
  VL_MEMREADY : Byte = 04;

{$I VLINE.INC}


VAR
  Schalter_MemReady, Speicher_VER_ADR : Byte;






procedure InitPorts;
begin

  // PORTA - Adressen Low
  DDRA:= %11111111;

  // PORTB - Adressen High
  DDRB:= %11111111;

  // PORTC - Daten
  DDRC:= %00000000;

{
D0,RXD
D1,TXD
D2,HOR     ; HORIZONTAL SYNC PULSE
D3,VER     ; VERTICAL SYNC PULSE
D4,G       ; VIDEO DAC ENABLE
D5,WE      ; SRAM WRITE ENABLE
D6,OE      ; SRAM OUTPUT ENABLE
D7,RAMPAGE ; SRAM PAGE CONTROL
}

  // PORTD - Control
  DDRD:= %11111111;


end InitPorts;








// VIDEO
isr TIMER1_COMPA;
begin

ASM;

;****************************************************************************
;****************************************************************************
;	VIDEO - Interruptroutine
;****************************************************************************
;****************************************************************************


; 2. Register in der Interruptroutine

.def memready = R17
.def VER_ADR  = R18


.equ 	port_adr_low	= porta
.equ 	port_adr_hig	= portb
.equ 	port_control	= portd

.equ 	ddr_dat		= ddrc

; *** port_control
.equ 	HOR		= 2			; Horizontale Synchronimpulse Pin 13
.equ 	VER		= 3			; Vertikale Synchronimpulse Pin 14
.equ 	G		= 4			; /CE fuer 74HC244
.equ 	WE		= 5			; /WE fuer SRAM
.equ 	OE		= 6			; /OE fuer SRAM
.equ 	RAMPAGE		= 7			;  RAMPAGE fuer SRAM


; ****************************************************************************************
; **** VIDEO RENDERING INTERRUPT / R2=SOUND FREQUENCY / RETURNS R25=MEMREADY
; ****************************************************************************************
VIDEO:


; bei 21,504 MHz
; HSP =  3800 nS / 46,5 nS =  81
; HBP =  1540 nS / 46,5 nS =  33
; HPX = 25600 nS / 46,5 nS = 550
; HFP =   640 nS / 46,5 nS =  14

; (3)

; EQUALIZE INTERRUPT LATENCY
; Angleichen der Interrupt Verborgenheit

	lds		R16,tcnt1l                    	;2
	cpi		R16,10	                       	;1
	brlo            LATFIX1	                      	;1/2

LATFIX1:
	cpi		R16,11	                	;1
	brlo           	LATFIX2		        	;1/2

LATFIX2:
	cpi		R16,12	                	;1
	brlo           	LATFIX3		                ;1/2

LATFIX3:
	cpi		R16,13	                	;1
	brlo	        LATFIX4		        	;1/2

LATFIX4:


; HORIZONTAL SYNC LOW
	cbi		port_control, HOR       	;2


; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
; |
; ---------------------------------------------
; ab hier 76 Zyklen HSP

; (0)

; PUSH REGISTERS TO STACK
	push     	zl				;2
	push           	zh				;2

        lds		memready, Schalter_MemReady     ;2
        lds		VER_ADR, Speicher_VER_ADR       ;2


; (8)

; SET POINTER TO VLINE TABLE
	ldi		zl,low(2*VLINE)	              	;1
	ldi		zh,high(2*VLINE)        	;1



; SET VLINE TABLE POSITION
	add		zl,yl				;1
	adc		zh,yh				;1


; READ VLINE TABLE BYTE
	lpm		R16,z		                ;3


; (15)


; INCREMENT VERTICAL ADDRESS
	sbrs         	R16,VL_INC_VA	        	;1/2
	inc		VER_ADR				;1


; RESET VERTICAL ADDRESS
	sbrc     	R16,VL_RESET			;1/2	; ueberspringen von ser, wenn
	ser		VER_ADR				;1		; Bit0=0 in R16


; INCREMENT VLINE COUNTER
	adiw      	yh:yl,1				;1


; (20)


; RESET VLINE COUNTER
	sbrc         	R16,VL_RESET			;1/2
	clr		yl				;1
	sbrc            R16,VL_RESET			;1/2
	clr		yh				;1


; (24)


; VERTICAL SYNC CONTROL
	sbrc    	R16,VL_VER_IMP	                ;1/2
	cbi		port_control,VER		;2
	sbrs           	R16,VL_VER_IMP	                ;1/2
	sbi		port_control,VER		;2

; (29)


; SET MEMREADY FLAG
	sbrc       	R16,VL_MEMREADY	               	;1/2
	clr		memready			;1
	sbrs            R16,VL_MEMREADY	                ;1/2
	ser		memready			;1


; (33)


; HORIZONTAL SYNC HIGH
	sbi		port_control,HOR		;2


; (35)


; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
;       |
; ---------------------------------------------
; ab hier 31 Zyklen HBP


; (0)



; ****************************************************************************************
; **** HORIZONTAL BACK PORCH = 36 CYCLES
; ****************************************************************************************

; SET VERTICAL ADDRESS
	sbrc          	R16,VL_OUT_VA	            	;1/2
	out		port_adr_hig,VER_ADR            ;1

; CONTROL ACTIVE LINES
	sbrs     	R16,VL_OUT_VA                   ;1/2
	rjmp            NOVID	                        ;2

; (5)

; SET SRAM TO IDLE
	sbi		port_control,WE			;2
	sbi		port_control,OE			;2

; RELEASE DATA BUS
	clr		R16				;1
	out		ddr_dat,R16			;1

; (11)

; SET SRAM TO READ
	cbi		port_control,OE			;2

; (13)


; ****************************************************************************************
; **** HORIZONTAL ACTIVE LINE = 512 CYCLES
; ****************************************************************************************

	; SET TO FRONT BUFFER
	in		R16,port_control		;1
	sbrc            R16,RAMPAGE			;1/2
	cbi		port_control,RAMPAGE            ;2
	sbrs            R16,RAMPAGE			;1/2
	sbi		port_control,RAMPAGE            ;2

; (19)

	; RESET HORIZONTAL ADDRESS
	clr		R16				;1
	out		port_adr_low,R16		;1

; (21)

	; TURN ON VIDEO DAC
	cbi		port_control,G			;2



; (23)


; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
;           |
; ---------------------------------------------
; ab hier 512 Zyklen HPX


	; PIXEL 1-10
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1


	; PIXEL 11-20
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 21-30
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 31-40
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 41-50
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 51-60
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 61-70
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 71-80
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 81-90
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 91-100
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 101-110
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 111-120
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 121-130
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 131-140
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 141-150
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1


	; PIXEL 151-160
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 161-170
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 171-180
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 181-190
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 191-200
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1


	; PIXEL 201-210
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 211-220
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1



	; PIXEL 221-230
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1


	; PIXEL 231-240
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1


	; PIXEL 241-250
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1


	; PIXEL 251-256
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1
	inc		R16				;1
	out		port_adr_low,R16	;1


; 256*2=512


; (0)

; TURN OFF VIDEO DAC
	sbi		port_control,G	        	;2


; (2)

; SET TO BACK BUFFER
	in		R16,port_control		;1
	sbrc            R16,RAMPAGE			;1/2
	cbi		port_control,RAMPAGE            ;2
	sbrs            R16,RAMPAGE			;1/2
	sbi		port_control,RAMPAGE            ;2

; (8)


; SET SRAM TO IDLE
	sbi		port_control,OE			;2


; ENABLE DATA BUS
	ser		R16				;1
	out		ddr_dat,R16			;1

; (12)



; ****************************************************************************************
; **** HORIZONTAL FRONT PORCH = 12 CYCLES
; ****************************************************************************************



; BLANKING LINES EXIT POINT
NOVID:

        sts		Schalter_MemReady,memready      ;2
        sts		Speicher_VER_ADR, VER_ADR       ;2


; POP REGISTERS FROM STACK
	pop		zh		                ;2
	pop		zl		                ;2


; (20)

; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
; |
; ---------------------------------------------


;19+ 47 + 42 +  512  +    35 = 655

; bei 22 MHz (45,45 ns) = 30 us (max. 31,75 us)


ENDASM;


end TIMER1_COMPA;





BEGIN

  InitPorts;

  Init_TIMER1_COMPA(CTC_WERT);

  EnableInts;

  LOOP

  ENDLOOP;


END VGAGEN2P.
