PROGRAM Z80HWE;

DEVICE = MEGA1284P;

{$I mega1284_Register.INC}



CONST
  // ProcClock = 20000000;       {Hertz}

  // Serial0 = Verbindung zum Peripherie-Adapter
  UBRR_VAL_S0 : Byte = 1;  // 625000 Baud
  // UBRR_VAL : Byte = 10;  // 115200 Baud

  // Serial1 = Verbindung zum VGA-Adapter
  UBRR_VAL_S1 : Byte = 1;  // 625000 Baud
  // UBRR_VAL : Byte = 10;  // 115200 Baud


  XON  : Byte = $11;
  XOFF : Byte = $13;



  // Befehlscode fuer IORQ
  IORQ_CONOUT     : Byte = 01;
  IORQ_CONIN      : Byte = 02;
  IORQ_CONST      : Byte = 03;
  IORQ_START_PROG : Byte = 04;
  IORQ_RAM2ARR    : Byte = 05;
  IORQ_ARR2RAM    : Byte = 06;
  IORQ_STD2UC     : Byte = 07;
  IORQ_RD_PC2ARR  : Byte = 08;
  IORQ_BDOS2RAM   : Byte = 09;
  IORQ_CCP2RAM    : Byte = 10;
  IORQ_WR_ARR2PC  : Byte = 11;
  IORQ_DISK2RAM   : Byte = 12;
  IORQ_LIST 	  : Byte = 13;
  IORQ_TIME 	  : Byte = 14;

  IORQ_LED_GN_Ein : Byte = 15;
  IORQ_LED_GN_Aus : Byte = 16;
  IORQ_LED_RT_Ein : Byte = 17;
  IORQ_LED_RT_Aus : Byte = 18;
  IORQ_LED_GE_Ein : Byte = 19;
  IORQ_LED_GE_Aus : Byte = 20;

  IORQ_CPM_Start  : Byte = 21;



  // Befehlscode fuer PA (Peripherie-Adapter)
  PA_CONST  : Byte = 01;
  PA_CONIN  : Byte = 02;
  PA_CONOUT : Byte = 03;
  PA_LIST   : Byte = 04;
  PA_SOUND  : Byte = 05;
  PA_WRITE  : Byte = 06;
  PA_READ   : Byte = 07;
  PA_STD    : Byte = 08;
  PA_Kommentar: Byte = 09;
  PA_Dummy    : Byte = 10;
  PA_CPM_Start: Byte = 11;
  // XON  : Byte = $11; -> 17
  // XOFF : Byte = $13; -> 19





  // Monitor-Steuerzeichen
  Monitor_leer   : Byte = 00;
  Monitor_links  : Byte = 01;
  Monitor_rechts : Byte = 02;





VAR

  // Globale Variablen

{
PA0 - A16_RAM   - A
PA1 - A17_RAM   - A
PA2 - A18_RAM   - A
PA3 - A19_RAM   - A
PA4 - CLK_Z80   - A
PA5 - OE_RAM    - A
PA6 - WE_RAM    - A
PA7 - G_RAMSW   - A
}
  A16_RAM[PortA,0]   : bit; // A
  A17_RAM[PortA,1]   : bit; // A
  A18_RAM[PortA,2]   : bit; // A
  A19_RAM[PortA,3]   : bit; // A
  CLK_Z80[PortA,4]   : bit; // A
  OE_RAM_A[PortA,5]  : bit; // E/A
  WE_RAM_A[PortA,6]  : bit; // E/A
  G_RAMSW[PortA,7]   : bit; // A

  OE_RAM_D[DDRA,5]    : bit; // A
  WE_RAM_D[DDRA,6]    : bit; // A


{
PB0 - M1_Z80    - E
PB1 - MREQ_Z80  - E
PB2 - IORQ_Z80  - E (INT2)
PB3 - LED_GN    - A
PB4 - LED_RT    - A
PB5 - MOSI      - E
PB6 - MISO      - E
PB7 - SCK       - E
}
  M1_Z80_E[PinB,0]    : bit; // E
  MREQ_Z80_E[PinB,1]  : bit; // E
  IORQ_Z80_E[PinB,2]  : bit; // E
  LED_GN[PortB,3]     : bit; // A
  LED_RT[PortB,4]     : bit; // A




{
PD0 - RXD0      - E
PD1 - TXD0      - A
PD2 - RXD1      - E
PD3 - TXD1      - A
PD4 - RD_Z80    - E
PD5 - WR_Z80    - E
PD6 - RESET_Z80 - A
PD7 - LED_GE    - A
}
  RD_Z80_E[PinD,4]   : bit; // E
  WR_Z80_E[PinD,5]   : bit; // E
  RESET_Z80[PortD,6] : bit; // A
  LED_GE[PortD,7]    : bit; // A



  Array_Pos : Word;

  ConsoleIn : Byte;

  Steuer_String : String[50];
  Anzeige_String : String[80];

  // Sektor-Puffer
  Sektor_Puffer : Array [0..127] of Byte;
  Sektor_Puffer_Zeiger : Word;

  // STD
  SECTOR, TRACK_L, TRACK_H, DISKNO : Byte;

  Datum_gesetzt : Boolean;



  // Monitor-Steuerung
  Monitor_verschieben : Byte;



CONST

// BIOS - Z80-Code
{$I CPD7_TXT.INC}


// BDOS - Z80-Code
{$I BDOS_TXT.INC}


// CCP - Z80-Code
{$I CCP_TXT.INC}


// RAM-Floppy
{$I DISK_TXT.INC}


{$I SYSTEM.INC}


{$I mega128_UART0_UART1_oI.INC}




// *** INITIALISIERUNG ***


procedure InitPorts;
begin

{                DIR  Pegel
PA0 - A16_RAM   - A     0
PA1 - A17_RAM   - A     0
PA2 - A18_RAM   - A     0
PA3 - A19_RAM   - A     0
PA4 - CLK_Z80   - A     0
PA5 - OE_RAM    - E     0
PA6 - WE_RAM    - E     0
PA7 - G_RAMSW   - A     0 - Durchschalten der Z80 Steuerleitungen
}
  DDRA  :=  %10011111;
  PortA :=  %01100000;

{
PB0 - M1_Z80    - E
PB1 - MREQ_Z80  - E
PB2 - IORQ_Z80  - E (INT2)
PB3 - LED_GN    - A
PB4 - LED_RT    - A
PB5 - MOSI      - E
PB6 - MISO      - E
PB7 - SCK       - E
}

  DDRB  :=  %00011000;
  PortB :=  %00000111;

{
PC0..PC7 = Daten (8)
}
  DDRC  :=  %00000000;
  PortC :=  %00000000;

{
PD0 - RXD0      - E
PD1 - TXD0      - A
PD2 - RXD1      - E
PD3 - TXD1      - A
PD4 - RD_Z80    - E
PD5 - WR_Z80    - E
PD6 - RESET_Z80 - A
PD7 - LED_GE    - A
}
  DDRD  :=  %11001010;
  PortD :=  %01000000;


end InitPorts;




// *** Z80 Hardware-Steuerprogramme ***



procedure sende_10_Takte;
var
  Zaehler : Byte;
begin

  for Zaehler:=1 to 100 do
    SetBit(CLK_Z80);
    NOP;
    NOP;
    ClearBit(CLK_Z80);
  endfor;

end sende_10_Takte;




procedure warte_solange_RD_High;
begin

  // warten bis /RD aktiv wird (Low)
  while BitHigh(RD_Z80_E) do

    if BitHigh(RD_Z80_E) then
      SetBit(CLK_Z80);
    endif;

     if BitHigh(RD_Z80_E) then
      ClearBit(CLK_Z80);
    endif;

  endwhile; // while BitHigh(RD_Z80_E) do

end warte_solange_RD_High;





procedure warte_solange_RD_Low;
begin

  // warten bis /RD inaktiv wird (High)
    while BitLow(RD_Z80_E) do
      SetBit(CLK_Z80);
      NOP;
      ClearBit(CLK_Z80);
    endwhile; // while BitLow(RD_Z80_E) do

end warte_solange_RD_Low;




procedure warte_solange_IORQ_High;
begin

  // warten bis /IORQ aktiv wird (Low)
  while BitHigh(IORQ_Z80_E) do

    if BitHigh(IORQ_Z80_E) then
      SetBit(CLK_Z80);
    endif;

     if BitHigh(IORQ_Z80_E) then
      ClearBit(CLK_Z80);
    endif;

  endwhile; // while BitLow(IORQ_Z80_E) do

end warte_solange_IORQ_High;




procedure warte_solange_IORQ_Low;
begin

  // warten bis /IORQ inaktiv wird (High)
  while BitLow(IORQ_Z80_E) do
    SetBit(CLK_Z80);
    NOP;
    ClearBit(CLK_Z80);
  endwhile; // while BitLow(IORQ_Z80_E) do

end warte_solange_IORQ_Low;




procedure lade_ARRAY_in_RAM;
begin


  // 3. Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  for Array_Pos:=0 to Laenge_CPD7_Array do

    warte_solange_RD_High;

    // 5. Einschreiben der Daten vom CPD5_ARR in den RAM
    // an der aktuellen Adresse (entspricht Array_Pos)

    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    // Anlegen der Z80-Daten
    PORTC := CPD7_ARR[Array_Pos];

    NOP;
    NOP;
    NOP;

    ClearBit(WE_RAM_A);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    SetBit(WE_RAM_A);

    // 6. Anlegen von NOP an den Datenport fuer die Z80
    PORTC := 0;  // NOP

    warte_solange_RD_Low;

    // Daten-Port auf Lesen
    DDRC  :=  %00000000;

  // 8. Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for Array_Pos:=0 to Laenge_CPD6_Array do }


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);


  // 11. Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);



end lade_ARRAY_in_RAM;














// laedt das Z80-BIOS in den RAM
// und startet es durch RESET
procedure Z80_RESET_und_URLOAD;
begin

  // 0. /RESET auf High setzen
  SetBit(RESET_Z80);

  sende_10_Takte;

  // 1. /RESET auf LOW setzen
  ClearBit(RESET_Z80);

  // 2a. Senden von min. 3 Takten (10 Takte)
  sende_10_Takte;

  // 2b. /RESET auf High setzen
  SetBit(RESET_Z80);

  lade_ARRAY_in_RAM;

  // 9. /RESET auf LOW setzen
  ClearBit(RESET_Z80);

  // 10a. Senden von min. 3 Takten (10 Takte)
  sende_10_Takte;


  // 10b. /RESET auf High setzen
  SetBit(RESET_Z80);

end Z80_RESET_und_URLOAD;








procedure IORQ_Ende;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;

end IORQ_Ende;








procedure Z80_CONOUT;
var
  IO_Wert : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist das auszugebende Zeichen
  warte_solange_IORQ_High;

  IO_Wert := PinC;

  // CONOUT an VGA-Adapter
  sende_Zeichen1_XON(IO_Wert);

  IORQ_Ende;

end Z80_CONOUT;








procedure Z80_LIST;
var
  IO_Wert : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist das auszugebende Zeichen
  warte_solange_IORQ_High;

  IO_Wert := PinC;

  // LIST an Peripherie-Adapter
  sende_Zeichen0_XON(PA_LIST);
  sende_Zeichen0_XON(IO_Wert);

  IORQ_Ende;

end Z80_LIST;








procedure Z80_CPM_Start;
begin

  IORQ_Ende;

  sende_Zeichen0_XON(PA_CPM_Start);

end Z80_CPM_Start;








procedure Z80_CONST;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  // CONST an Peripherie-Adapter
  sende_Zeichen0_XON(PA_CONST);

  while BitLow(RXC0) do

  endwhile;

  ConsoleIn:=UDR0;

  DDRC:=%11111111;
  PortC:=ConsoleIn;

  IORQ_Ende;

  SER0_warten:=false;
  ConsoleIn:=0;

end Z80_CONST;










procedure Z80_CONIN;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  // CONIN an Peripherie-Adapter
  sende_Zeichen0_XON(PA_CONIN);

  while BitLow(RXC0) do

  endwhile;

  ConsoleIn:=UDR0;

  DDRC:=%11111111;
  PortC:=ConsoleIn;

  IORQ_Ende;

  SER0_warten:=false;

end Z80_CONIN;












// Start eines beliebigen Testprogramms des uC
// durch die Z80
procedure START_PROG;
var
  b1 : Byte;
  b2 : Byte;
  b3 : Byte;
  Adresse : Word;
begin
  Anzeige_String:='';
  Adresse:=0;

  for b1:=1 to 8 do
    for b2:=1 to 16 do
      b3:=Sektor_Puffer[Adresse];
      Inc(Adresse);
      ByteToString(b3);
      Anzeige_String:=Anzeige_String+Write_String;
      Anzeige_String:=Anzeige_String+' ';
    endfor;
  //  CONSOLE_CONOUT_Anzeige_String;
  endfor;

 // CONSOLE_CONOUT($0D);
 // CONSOLE_CONOUT($0A);

  IORQ_Ende;

end START_PROG;









// Kopieren des RAMs ab Adresse in den Sektor_Puffer
procedure START_RAM2ARR;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
  RAM_PORT : Byte;
begin


  // warten bis /IORQ inaktiv wird (High)


// warte_solange_IORQ_Low
ASM;

M_ASM_warte_solange_IORQ_Low_R2A1:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_Low_R2A1

ENDASM;


  // Das naechste Byte ist das auszugebende Zeichen

// warte_solange_IORQ_High
ASM;

M_ASM_warte_solange_IORQ_High_R2A2:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_High_A_R2A2

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_High_R2A2


M_ASM_warte_solange_IORQ_High_A_R2A2:


ENDASM;



  RAM_ADR := PinC;



// warte_solange_IORQ_Low
ASM;

M_ASM_warte_solange_IORQ_Low_R2A3:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_Low_R2A3

ENDASM;



  // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl



// warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_R2A4:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A_R2A4

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_R2A4

M_ASM_warte_solange_RD_High_A_R2A4:

ENDASM;



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_R2A5:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_Low_R2A5

ENDASM;



  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird der RAM gelesen ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := RAM_ADR AND %00001111;
  RAM_PORT := PortA AND %11110000;
  RAM_ADR := RAM_ADR OR RAM_PORT;
  PortA := RAM_ADR;


  for SP_Zaehler:=0 to 127 do


 // warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_R2A6:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A_R2A6

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_R2A6

M_ASM_warte_solange_RD_High_A_R2A6:

ENDASM;



    // Daten-Port auf Lesen
    DDRC  :=  %00000000;

    ClearBit(OE_RAM_A);

    NOP;
    NOP;
    NOP;

    RAM_Wert := PinC;
    Sektor_Puffer[SP_Zaehler]:=RAM_Wert;

    SetBit(OE_RAM_A);

    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;


    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_R2A7:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_Low_R2A7

ENDASM;



  // 8. Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for Array_Pos:=0 to Laenge_CPD6_Array do }


  RAM_ADR := PortA AND %11110000;
  PortA := RAM_ADR;



  // *** Einschreiben von RET ($C9) an Z80




// warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_R2A8:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A_R2A8

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_R2A8

M_ASM_warte_solange_RD_High_A_R2A8:

ENDASM;



  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_R2A9:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_Low_R2A9

ENDASM;


  // Daten-Port auf Lesen
  DDRC  :=  %00000000;

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end START_RAM2ARR;








// Kopieren des Sektor_Puffer in den RAMs ab DMA-Adresse
procedure START_ARR2RAM;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
  RAM_PORT : Byte;
begin



  // warten bis /IORQ inaktiv wird (High)
// warte_solange_IORQ_Low
ASM;

M_ASM_warte_solange_IORQ_Low_A2R1:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_Low_A2R1

ENDASM;



  // Das naechste Byte ist das auszugebende Zeichen
// warte_solange_IORQ_High
ASM;

M_ASM_warte_solange_IORQ_High_A2R2:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_High_A_A2R2

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_High_A2R2

M_ASM_warte_solange_IORQ_High_A_A2R2:

ENDASM;



  RAM_ADR := PinC;



// warte_solange_IORQ_Low
ASM;

M_ASM_warte_solange_IORQ_Low_A2R3:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinB, 2
        RJMP       M_ASM_warte_solange_IORQ_Low_A2R3

ENDASM;



  // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl

// warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_A2R4:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A_A2R4

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A2R4

M_ASM_warte_solange_RD_High_A_A2R4:

ENDASM;



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_A2R5:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_Low_A2R5

ENDASM;


  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird der RAM gelesen ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := RAM_ADR AND %00001111;
  RAM_PORT := PortA AND %11110000;
  RAM_ADR := RAM_ADR OR RAM_PORT;
  PortA := RAM_ADR;


  for SP_Zaehler:=0 to 127 do




 // warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_A2R6:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A_A2R6

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A2R6


M_ASM_warte_solange_RD_High_A_A2R6:


ENDASM;


    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    RAM_Wert := Sektor_Puffer[SP_Zaehler];
    PortC := RAM_Wert;

    ClearBit(WE_RAM_A);

    NOP;
    NOP;
    NOP;

    SetBit(WE_RAM_A);

    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_A2R7:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_Low_A2R7

ENDASM;



  // Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for Array_Pos:=0 to Laenge_CPD6_Array do }


  RAM_ADR := PortA AND %11110000;
  PortA := RAM_ADR;


  // *** Einschreiben von RET ($C9) an Z80



// warte_solange_RD_High
ASM;

M_ASM_warte_solange_RD_High_A2R8:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A_A2R8

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIC       PinD, 4
        RJMP       M_ASM_warte_solange_RD_High_A2R8

M_ASM_warte_solange_RD_High_A_A2R8:

ENDASM;


  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET



// warte_solange_RD_Low
ASM;

M_ASM_warte_solange_RD_Low_A2R9:

; SetBit(CLK_Z80)
        SBI        PORTA, 4

        NOP

; ClearBit(CLK_Z80)
        CBI        PORTA, 4

        SBIS       PinD, 4
        RJMP       M_ASM_warte_solange_RD_Low_A2R9

ENDASM;


  // Daten-Port auf Lesen
  DDRC  :=  %00000000;

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end START_ARR2RAM;









// Kopieren des BDOS in den RAM
procedure START_BDOS2RAM;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

 // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl

  warte_solange_RD_High;
  warte_solange_RD_Low;

  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird in den RAM geschrieben ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := PortA AND %11110000;
  PortA := RAM_ADR;


  for SP_Zaehler:=0 to Laenge_BDOS_Array do

    warte_solange_RD_High;


    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    RAM_Wert := BDOS_ARR[SP_Zaehler];
    PortC := RAM_Wert;

    ClearBit(WE_RAM_A);

    NOP;
    NOP;
    NOP;

    SetBit(WE_RAM_A);


    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP

    warte_solange_RD_Low;

  // Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for SP_Zaehler:=0 to Laenge_BDOS_Array do }



  // *** Einschreiben von RET ($C9) an Z80

  warte_solange_RD_High;

  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET

  warte_solange_RD_Low;

  // Daten-Port auf Lesen
  DDRC  :=  %00000000;

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end START_BDOS2RAM;






// Kopieren des CCP in den RAM
procedure START_CCP2RAM;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;


  // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl

  warte_solange_RD_High;
  warte_solange_RD_Low;


  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird in den RAM geschrieben ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := PortA AND %11110000;
  PortA := RAM_ADR;


  for SP_Zaehler:=0 to Laenge_CCP_Array do

    warte_solange_RD_High;


    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    RAM_Wert := CCP_ARR[SP_Zaehler];
    PortC := RAM_Wert;

    ClearBit(WE_RAM_A);

    NOP;
    NOP;
    NOP;

    SetBit(WE_RAM_A);


    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP

    warte_solange_RD_Low;

  // Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for SP_Zaehler:=0 to Laenge_BDOS_Array do }



  // *** Einschreiben von RET ($C9) an Z80

  warte_solange_RD_High;

  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET

  warte_solange_RD_Low;

  // Daten-Port auf Lesen
  DDRC  :=  %00000000;

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end START_CCP2RAM;






// Kopieren des Disk-Images in den RAM
procedure COPY_DISK_TO_RAM;
var
  SP_Zaehler : Word;
  RAM_Wert : Byte;
  RAM_ADR : Byte;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;


  // Der naechste M1-Zyklus ist der "JMP (HL)"-Befehl

  warte_solange_RD_High;
  warte_solange_RD_Low;


  // Der naechste M1-Zyklus wird abgefangen

  // *** nun wird in den RAM geschrieben ***

  // Abschalten der /RD und /WR Signale der Z80 an den RAM
  SetBit(G_RAMSW);


  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Ausgabe
  SetBit(OE_RAM_D);
  SetBit(WE_RAM_D);

  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);


  RAM_ADR := PortA AND %11110000;
  PortA := RAM_ADR OR 00000001;


  for SP_Zaehler:=0 to Laenge_DISK_Array do

    warte_solange_RD_High;

    // Daten-Port auf Schreiben
    DDRC  :=  %11111111;

    RAM_Wert := DISK_ARR[SP_Zaehler];
    PortC := RAM_Wert;

    ClearBit(WE_RAM_A);

    NOP;
    NOP;
    NOP;

    SetBit(WE_RAM_A);

    // Anlegen von NOP an den Datenport fuer die Z80
    PortC := 0;  // NOP

    warte_solange_RD_Low;

  // Wiederholen bis das ganze ARRAY in den RAM geladen wurde
  endfor; { for SP_Zaehler:=0 to Laenge_BDOS_Array do }



  // *** Einschreiben von RET ($C9) an Z80

  warte_solange_RD_High;

  // Daten-Port auf Schreiben
  DDRC  :=  %11111111;

  // Anlegen von RET an den Datenport fuer die Z80
  PORTC := $C9;  // RET

  warte_solange_RD_Low;

  // Daten-Port auf Lesen
  DDRC  :=  %00000000;




  RAM_ADR := PortA AND %11110000;
  PortA := RAM_ADR;



  // OE und WE aktivieren
  SetBit(OE_RAM_A);
  SetBit(WE_RAM_A);

  // OE und WE DIR Eingabe
  ClearBit(OE_RAM_D);
  ClearBit(WE_RAM_D);

  // Zuschalten der /RD und /WR Signale der Z80 an den RAM
  ClearBit(G_RAMSW);

end COPY_DISK_TO_RAM;











procedure STD_an_uC;
begin

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist der SECTOR
  warte_solange_IORQ_High;

  SECTOR := PinC;


  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist der TRACK_L
  warte_solange_IORQ_High;

  TRACK_L := PinC;


  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist der TRACK_H
  warte_solange_IORQ_High;

  TRACK_H := PinC;


  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  // Das naechste Byte ist der DISKNO
  warte_solange_IORQ_High;

  DISKNO := PinC;


  IORQ_Ende;


end STD_an_uC;







// liest die Bytes aus der SteuerString in den DATE und TIME-Puffer
procedure Steuer_String_To_TIME;
var
  b1 : Byte;
  b2 : Byte;
  c1 : Char;
begin


  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;


  // Date Low-Teil
  c1:=Steuer_String[4];
  Write_String:=c1;
  c1:=Steuer_String[5];
  Write_String:=Write_String+c1;
  b1:=StringToByte;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  // Date High-Teil
  c1:=Steuer_String[2];
  Write_String:=c1;
  c1:=Steuer_String[3];
  Write_String:=Write_String+c1;
  b1:=StringToByte;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  // HOUR
  c1:=Steuer_String[6];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[7];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  // MIN
  c1:=Steuer_String[8];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[9];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  // SEC
  c1:=Steuer_String[10];
  b1:=CharToByte(c1);
  b1:=b1-$30;
  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);
  c1:=Steuer_String[11];
  b2:=CharToByte(c1);
  b2:=b2-$30;
  b1:=b1+b2;


  // Das naechste Byte ist das einzulesende Zeichen
  warte_solange_IORQ_High;

  DDRC:=%11111111;
  PortC:=b1;

  // warten bis /IORQ inaktiv wird (High)
  warte_solange_IORQ_Low;

  DDRC:=%00000000;


  Datum_gesetzt:=true;

end Steuer_String_To_TIME;







procedure UP_SET_TIME;
var
  LeseSchleife : Word;
begin

  Write_String:='d';
 // WriteLn_SER0;

  Datum_gesetzt:=false;

  LeseSchleife:=60000;

    while LeseSchleife>5 do

      if BitHigh(RXC0) then

      //  Steuer_String_lesen;

        if Datum_gesetzt then
          LeseSchleife:=2;  // Ende
        else
          LeseSchleife:=60000;
        endif; { if Datum_gesetzt then }

      endif;

      Dec(LeseSchleife);
    endwhile;

  if LeseSchleife>2 then
    Write_String:='x10';
   // WriteLn_SER0;
  endif;

end UP_SET_TIME;







// ********************************************







procedure warte_kurz;
begin

  NOP;
  NOP;
  NOP;
  NOP;
  NOP;

end warte_kurz;





procedure Kommentar_an_PA_senden;
var
  b1 : Byte;
  b2 : Byte;
  c1 : Char;

begin

  sende_Zeichen0_XON(PA_Kommentar);

  warte_kurz;
  warte_kurz;
  warte_kurz;

  b1:=Length(Write_String);

  sende_Zeichen0(b1);

  for b2:=1 to b1 do

    warte_kurz;
    c1:=Write_String[b2];
    b1:=CharToByte(c1);
    sende_Zeichen0(b1);

  endfor; { for Sektor_Puffer_Zeiger:=0 to 127 do }

end Kommentar_an_PA_senden;







// ******************************************
// *** STD, READ und WRITE fuer PA und PC ***
// ******************************************








procedure STD_an_PA_senden;
begin

  sende_Zeichen0_XON(PA_STD);

  sende_Zeichen0_XON(SECTOR);

  sende_Zeichen0_XON(TRACK_H);

  sende_Zeichen0_XON(TRACK_L);

  sende_Zeichen0_XON(DISKNO);

end STD_an_PA_senden;








procedure READ_Sektor_von_PA_lesen;
begin

  sende_Zeichen0_XON(PA_READ);


  for Sektor_Puffer_Zeiger:=0 to 127 do


// wurde ein Byte durch die UART empfangen, so ist das RXC0-Bit gesetzt
// ist RXC0-Bit = 0, dann ist der Empfangs-Puffer leer
ASM;

M_ASM_warte_solange_READ_Sektor:

        LDS        R16, UCSR0A
        BST        R16, 7
        BRTC       M_ASM_warte_solange_READ_Sektor

ENDASM;

    SER0_Zeichen:=UDR0;
    Sektor_Puffer[Sektor_Puffer_Zeiger]:=SER0_Zeichen;

  endfor; { for Sektor_Puffer_Zeiger:=0 to 127 do }

end READ_Sektor_von_PA_lesen;








procedure WRITE_Sektor_an_PA_senden;
var
  b1 : Byte;
begin

  sende_Zeichen0_XON(PA_WRITE);

  // Abschalten Interrupt am PA
  sende_Zeichen0_XON(PA_Dummy);

  for Sektor_Puffer_Zeiger:=0 to 127 do

    warte_kurz;
    b1:=Sektor_Puffer[Sektor_Puffer_Zeiger];
    sende_Zeichen0(b1);

  endfor; { for Sektor_Puffer_Zeiger:=0 to 127 do }

end WRITE_Sektor_an_PA_senden;




// ********************************************













procedure Auswerten_IORQ;
var
  IO_Wert : Byte;
begin

  if BitLow(WR_Z80_E) then

    // Daten-Port auf Lesen
    DDRC  :=  %00000000;

    IO_Wert := PinC;

    case IO_Wert of


      IORQ_CONOUT      : Z80_CONOUT;
      |

      IORQ_CONIN       : Z80_CONIN;
      |

      IORQ_CONST       : Z80_CONST;
      |

      IORQ_START_PROG  : START_PROG;
      |

      IORQ_RAM2ARR     : START_RAM2ARR;
      |

      IORQ_ARR2RAM     : START_ARR2RAM;
      |

      IORQ_STD2UC      : STD_an_uC;
      |

      IORQ_RD_PC2ARR   : STD_an_PA_senden;
                         READ_Sektor_von_PA_lesen;
      |

      IORQ_BDOS2RAM    : START_BDOS2RAM;
      |

      IORQ_CCP2RAM     : START_CCP2RAM;
      |

      IORQ_WR_ARR2PC   : STD_an_PA_senden;
                         WRITE_Sektor_an_PA_senden;
      |

      IORQ_DISK2RAM    : COPY_DISK_TO_RAM;
      |

      IORQ_LIST        : Z80_LIST;
      |

      IORQ_TIME        : UP_SET_TIME;
      |

      IORQ_LED_GN_Ein  : SetBit(LED_GN);
                         IORQ_Ende;
      |

      IORQ_LED_GN_Aus  : ClearBit(LED_GN);
                         IORQ_Ende;
      |

      IORQ_LED_RT_Ein  : SetBit(LED_RT);
                         IORQ_Ende;
      |

      IORQ_LED_RT_Aus  : ClearBit(LED_RT);
                         IORQ_Ende;
      |

      IORQ_LED_GE_Ein  : SetBit(LED_GE);
                         IORQ_Ende;
      |

      IORQ_LED_GE_Aus  : ClearBit(LED_GE);
                         IORQ_Ende;
      |

      IORQ_CPM_Start   : Z80_CPM_Start;
      |

    else
      SetBit(LED_RT);
      SetBit(LED_GE);
      SetBit(LED_GN);
    endcase; { case IO_Wert of }

  endif; // if BitLow(WR_Z80) then

end Auswerten_IORQ;





BEGIN

ASM;

.equ UBRR0 = UBRR0L

ENDASM;


  InitPorts;

  // kurzes Blinken zum Start
  SetBit(LED_RT);
  wait_ms(100);
  ClearBit(LED_RT);
  wait_ms(100);


  INIT_SER0_PORT;
  INIT_SER1_PORT;

  ConsoleIn:=0;
  Steuer_String:='';
  Datum_gesetzt:=false;


  // sende_Zeichen0(64);  // @  = Ruecksetzen der Hardware
  SER0_warten:=false;
  SER1_warten:=false;

  // kurzes Blinken zum Start
  SetBit(LED_GN);
  wait_ms(100);
  ClearBit(LED_GN);
  wait_ms(100);


  // Laden des Z80-Bios
  Z80_RESET_und_URLOAD;


  SetBit(LED_GE);
  wait_ms(100);
  ClearBit(LED_GE);
  wait_ms(100);



{
Write_String:='Hallo Welt !!!';
Kommentar_an_PA_senden;
}


  // Endlosschleife fuer Takterzeugung
  // Taktfrequenz etwa 2 MHz
  LOOP


// 1. Taktperiode

    SetBit(CLK_Z80);      // 2 Takte

ASM;

    LDS      R16, UCSR0A
    SBRC     R16, 7
    // Serial0 vom Periperie-Adapter (XON, m, m)
    CALL     Serial0_abfragen;

ENDASM;

    ClearBit(CLK_Z80);    // 2 Takte

ASM;

    SBIS     PinB, 2
    CALL     AUSWERTEN_IORQ

ENDASM;




// 2. Taktperiode



    SetBit(CLK_Z80);      // 2 Takte

ASM;

    LDS      R16, UCSR1A
    SBRC     R16, 7
    // Rueckmeldung vom VGA (von dort kommt nur XON)
    CALL     Serial1_abfragen;

ENDASM;

    ClearBit(CLK_Z80);  // 2 Takte

ASM;

    SBIS     PinB, 2
    CALL     AUSWERTEN_IORQ

ENDASM;



    // LDS  = 1 Takt
    // SBRC = 1 Takt
    // SBIS = 1 Takt
    // RJMP = 2 Takte
    // SetBit(CLK_Z80) = 2 Takte
    // ClearBit(CLK_Z80) = 2 Takte

    // 20 MHz / 9 = 2,1 MHz



  ENDLOOP;  // RJMP - 2 Takte


END Z80HWE.
