

ASM;

.equ 	true		= 0xFF
.equ 	false		= 0x00


; Verwaltung der Ports
; ---------------------

; Verwaltung der Ports
.equ 	port_adr_low	= porta
.equ 	port_adr_hig	= portb
.equ 	port_dat	= portc
.equ 	ddr_dat	        = ddrc
.equ 	port_control	= portd




; *** port_control_d ***
.equ 	HSYNC		= 2			; Horizontale Synchronimpulse Pin 13
.equ 	VSYNC		= 3			; Vertikale Synchronimpulse Pin 14
.equ 	G		= 4			; /CE fuer 74HC244
.equ 	WE		= 5			; /WE fuer SRAM
.equ 	OE		= 6			; /OE fuer SRAM



; VERTICAL LINE CONTROL
.equ 	VL_RESET	= 0			; Ruecksetzen von VL
.equ 	VL_VER_IMP	= 1			;
.equ 	VL_OUT_VA	= 2			;
.equ 	VL_INC_VA	= 3			;
.equ 	VL_UP_ON	= 4			; Text und Grafik UP aktivieren
.equ 	VL_UP_OFF	= 5			; Text und Grafik UP abschalten



; Hier beginnt der VIDEO-Code

	lds        R16,tcnt1l
	cpi        R16,12
	brlo       LATFIX1

LATFIX1:
	cpi        R16,13
	brlo       LATFIX2

LATFIX2:
	cpi       R16,14
	brlo      LATFIX3

LATFIX3:
	cpi       R16,15
	brlo      LATFIX4

LATFIX4:

; HORIZONTAL SYNC auf LOW
	cbi	  port_control, HSYNC


; Sichern weiterer Register
	push	  R17

	push	  zl
	push	  zh
	push	  yl
	push	  yh


; Laden des letzten Standes wichtiger Register
	lds	  yl, Wert_yl
	lds	  yh, Wert_yh

	lds	  R17, Wert_R17



; setzen des Zeigers auf die VLINE-Tabelle
	ldi	  zl,low(2*VLINE)
	ldi	  zh,high(2*VLINE)



; berechnen der aktuellen Tabellen-Position
	add	  zl,yl
	adc	  zh,yh



; lesen des Bytes aus der VLINE-Tabelle in R16
	lpm	  R16,z



; ist VL_INC_VA gesetzt, so wird die vertikale Adresse erhoeht
	sbrs	  R16,VL_INC_VA
	inc	  R17



; ist VL_RESET gesetzt, so wird die vertikale Adresse auf 0FFH gesetzt
	sbrc	  R16,VL_RESET
	ser	  R17



; inkrementieren des Adress-Offsets der VLINE-Tabelle
; fuer den naechsten Durchlauf
	adiw	  yh:yl,1



; ist VL_RESET gesetzt, so wird auch der Adress-Offset der
; VLINE-Tabelle auf 0 gesetzt
	sbrc	  R16,VL_RESET
	clr	  yl
	sbrc	  R16,VL_RESET
	clr	  yh



; Abspeichern des Wertes fuer den Adress-Offset
; fuer den naechsten Durchlauf
      	sts	  Wert_yl, yl
	sts	  Wert_yh, yh



; ist VL_VER_IMP gesetzt, so wird VSYNC auf 0 gesetzt
; ansonsten wird es auf 1 gesetzt
	sbrc	  R16, VL_VER_IMP
	cbi	  port_control, VSYNC
	sbrs	  R16, VL_VER_IMP
	sbi	  port_control, VSYNC



; HORIZONTAL SYNC wird auf HIGH gesetzt
	sbi	  port_control, HSYNC



; ist VL_OUT_VA gesetzt, so wird die vertikale Adresse ausgegeben
	sbrc	  R16,VL_OUT_VA
	out	  port_adr_hig,R17



; Abspeichern des Wertes fuer die vertikale Adresse
; fuer den naechsten Durchlauf
	sts	  Wert_R17, R17



; ist VL_OUT_VA gesetzt, so befinden wir uns im unsichtbaren Teil
; der Anzeige und es wird auf NOVID gesprungen
; Ansonsten geht es im Progeramm weiter
	sbrs	  R16,VL_OUT_VA
	rjmp      NOVID


        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP



; Abschalten des SRAMs
; WE auf 1 setzen
;	sbi       port_control,WE        ;2
; OE auf 1 setzen
;	sbi       port_control,OE        ;2



; SRAM auf Lesen stellen
	cbi       port_control,OE        ;2


; Daten-Bus auf Eingang stellen
	clr       R16                    ;1
	out       ddr_dat,R16            ;1




; Ruecksetzen der horizontalen Adresse
	clr       R16                    ;1
	out       port_adr_low,R16       ;1



; Einschalten des Video-DAC mit den Farbsignalen
	cbi       port_control,G         ;2



; ab hier wird der Inhalt des SRAMs an den Video-DAC ausgegeben

; bei 20 MHz und 256 Pixeln (512 Takte) sind das genau 25,6 us (25,4 us laut VGA)



	; PIXEL 1-10
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 11-20
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 21-30
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 31-40
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 41-50
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 51-60
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 61-70
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 71-80
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 81-90
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 91-100
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1





	; PIXEL 101-110
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1


	; PIXEL 111-120
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 121-130
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 131-140
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 141-150
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 151-160
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 161-170
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 171-180
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 181-190
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 191-200
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1






	; PIXEL 201-210
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1


	; PIXEL 211-220
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 221-230
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 231-240
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 241-250
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1



	; PIXEL 251-256
       	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1
	inc       R16                    ;1
	out       port_adr_low,R16       ;1




; Abschalten des Video-DAC mit den Farbsignalen
	sbi       port_control,G         ;2


; Abschalten des SRAMs (Standby)
	sbi       port_control, OE       ;2


        rjmp      VIDEO_END

; -----------------------------------------------



; Einsprungpunkt fuer die Austastluecke (keine Ausgabe der Videosignale)
NOVID:

; ist VL_UP_ON gesetzt, so werden alle Register fuer die weitere
; Abarbeitung der Text und Grafik Unterprogramme geladen
; Ansonsten geht es zu NOVID_OFF

	sbrs	  R16,VL_UP_ON
	rjmp      NOVID_OFF

; VL_UP_ON
; --------

; 1. Aufraumen des Stacks - Sichern aller Register,
;                           die auf den Stack geladen wurden

; 1.1. die auf dem Stack gesicherten Register - aus dem Programm


	pop       yh
	pop       yl
	pop       zh
	pop       zl
	pop       r17

; 1.2. die auf dem Stack gesicherten Register - aus der ISR

	pop       r16
	sts	  Endlos_SREG, R16

	pop       r16
	sts	  Endlos_R16, R16


; 1.3. der auf dem Stack gesicherte ProgramCounter (2 Byte)


        pop       r16

        pop       r16



; 2. Sichern der IO-Register und ihrer Werte fuer Endlos


        in        R16, port_control
        sts       Endlos_port_control, R16




; 3. Laden der IO-Register und ihrer Werte fuer Programm

	lds	  R16, Save_port_adr_low
	out       port_adr_low, R16

	lds	  R16, Save_port_adr_hig
	out       port_adr_hig, R16

	lds	  R16, Save_port_dat
	out       port_dat, R16

	lds	  R16, Save_ddr_dat
	out       ddr_dat, R16

	lds	  R16, Save_port_control
	out       port_control, R16


; ist UP_SCHLEIFE_AKTIV aktiv
        lds	  R16, UP_SCHLEIFE_AKTIV
        cpi       R16, false

        breq      NOVID_T1


; Interrupt Timer 1 aus
        LDI        R16, 0x00
        STS        TIMSK1, R16

; Interrupt Timer 3 ein
        LDI        R16, 0x02
        STS        TIMSK3, R16


NOVID_T1:

; 4. Laden des Stacks

        lds	  R16, Save_SP01
        push      r16

        lds	  R16, Save_SP02
        push      r16

        lds	  R16, Save_R16
        push      r16

        lds	  R16, Save_SREG
        push      r16


        rjmp      ISR_END


NOVID_OFF:

; ist VL_UP_OFF gesetzt, so werden alle Register fuer die
; Abarbeitung der Text und Grafik Unterprogramme abgespeichert
; und es wird wieder die kleine Entlosschleife geladen
; Ansonsten geht es zu VIDEO_END

	sbrs	  R16,VL_UP_OFF
	rjmp      VIDEO_END

; VL_UP_OFF
; ---------

; 1. Aufraumen des Stacks - Sichern aller Register,
;                           die auf den Stack geladen wurden

; 1.1. die auf dem Stack gesicherten Register - aus dem Programm


	pop       yh
	sts	  Save_R29, R29

	pop       yl
	sts	  Save_R28, R28

	pop       zh
	sts	  Save_R31, R31

	pop       zl
	sts	  Save_R30, R30

	pop       r17
	sts	  Save_R17, R17


; 1.2. die auf dem Stack gesicherten Register - aus der ISR

	pop       r16
	sts	  Save_SREG, R16

	pop       r16
	sts	  Save_R16, R16


; 1.3. der auf dem Stack gesicherte PC (2 Byte)

        pop       r16
        sts       Save_SP02, R16

        pop       r16
        sts       Save_SP01, R16


; 2. Sichern der IO-Register und ihrer Werte vom Programm

        in        R16, port_adr_low
        sts       Save_port_adr_low, R16

        in        R16, port_adr_hig
        sts       Save_port_adr_hig, R16

        in        R16, port_dat
        sts       Save_port_dat, R16

        in        R16, ddr_dat
        sts       Save_ddr_dat, R16

        in        R16, port_control
        sts       Save_port_control, R16



; 3. Laden der IO-Register und ihrer Werte fuer Endlosschleife

	lds	  R16, Endlos_port_control
	out       port_control, R16



; Daten-Bus auf Ausgang stellen
	clr       R16
	out       ddr_dat, R16




; 4. Laden des Stacks

        lds	  R16, Endlos_SP01
        push      r16

        lds	  R16, Endlos_SP02
        push      r16

        lds	  R16, Endlos_R16
        push      r16

        lds	  R16, Endlos_SREG
        push      r16


        rjmp      ISR_END




VIDEO_END:

	pop       yh
	pop       yl
	pop       zh
	pop       zl

	pop       r17

ISR_END:



ENDASM;



