
// Timer / Counter 0

procedure InitTimer0;
begin
  // Timer/Counter Control Register 0
//  TCCR0:=%00000000;    // kei  Takt (Timer/Zaehler ist gestoppt)
  TCCR0:=%00000001;    // kein Vorteiler
//  TCCR0:=%00000010;    // Vorteiler = Takt/8
//  TCCR0:=%00000011;    // Vorteiler = Takt/64
//  TCCR0:=%00000100;    // Vorteiler = Takt/256
//  TCCR0:=%00000101;    // Vorteiler = Takt/1024
//  TCCR0:=%00000110;    // Externe Taktquelle an T0 Pin. Takt an der fallenden Flanke
//  TCCR0:=%00000111;    // Externe Taktquelle an T0 Pin. Takt an der steigenden Flanke

  // Timer/Counter Interrupt Mask Register
  // Bit 0 – TOIE0: Timer/Counter0 Overflow Interrupt Enable
  TIMSK:=%00000001;    // TOIE0=1 -> Interrupt bei Timer Overflow
end InitTimer0;



// Timer / Counter 1






procedure Init_TIMER1_COMPA(Wert : Word);
VAR
  B_wert_L : Byte;
  B_Wert_H : Byte;
begin
{
; CTC Modus (Clear Timer on Compare Match mode)
; ---------------------------------------------

; genutzt wird der 16-Bit Timer1

; TCCR1B = Timer/Counter control Register B des Timers1
; | ICNC1 | ICES1 | leer | WGM13 | WGM12 | CS12 | CS11 | CS10 |
; CS12  CS11  CS10
;   0     0     1  = CPU-Takt ohne Teiler
;
; WGM12 = // CTC-Modus, TOP = OCR1A
;
; Timer 1, ein 16-Bit-Timer, wird mit einem Vorteiler von 1 betrieben.
; Dadurch wird erreicht, dass der Timer mit höchster Zeitauflösung arbeiten kann.
; Bei jedem Ticken des Systemtaktes von 4 MHz wird auch der Timer um 1 erhöht.
; Zusätzlich wird noch das WGM12-Bit bei der Konfiguration gesetzt.
; Dadurch wird der Timer in den CTC-Modus gesetzt.
; Dabei wird der Inhalt des Timers hardwaremäßig mit dem Inhalt des OCR1A-Registers verglichen.
; Stimmen beide überein, so wird der Timer auf 0 zurückgesetzt und im nächsten Taktzyklus ein OCIE1A-Interrupt ausgelöst.
; Dadurch ist es möglich, exakt die Anzahl an Taktzyklen festzulegen, die von einem Interrupt zum nächsten vergehen sollen.
}

  // Bit 7:6 – COM1A1:0: Compare Output Mode for channel A
  // Bit 5:4 – COM1B1:0: Compare Output Mode for channel B
  // Normal port operation, OC1A/OC1B disconnected.
  // 0 0 0 0

//  TCCR1A:=%00000000;


  // Timer/Counter Control Register 0
  // Bit 4:3 – WGM13:2: Waveform Generation Mode
  //   0 1 - CTC - OCR1A
//  TCCR1B:=%00001000;    // kein Takt (Timer/Zaehler ist gestoppt)
  TCCR1B:=%00001001;    // kein Vorteiler
//  TCCR1B:=%00001010;    // Vorteiler = Takt/8
//  TCCR1B:=%00001011;    // Vorteiler = Takt/64
//  TCCR1B:=%00001100;    // Vorteiler = Takt/256
//  TCCR1B:=%00001101;    // Vorteiler = Takt/1024
//  TCCR1B:=%00001110;    // Externe Taktquelle an T0 Pin. Takt an der fallenden Flanke
//  TCCR1B:=%00001111;    // Externe Taktquelle an T0 Pin. Takt an der steigenden Flanke

{
; Einstellen der Interrupt-Zeit
}
  B_Wert_H:=High(Wert);
  B_Wert_L:=Low(Wert);

  OCR1AH:=B_Wert_H;
  OCR1AL:=B_Wert_L;

{
; TIMSK
; | TICIE1 | OCIE1A | OCIE1B | TOIE1 |
;
; TICIE1 - Timer 1 Input Capture Interrupt Enable
; OCIE1A - Timer 1 Output Compare A Match Interrupt Enable
; OCIE1B - Timer 1 Output Compare B Match Interrupt Enable
; TOIE1  - Timer 1 Overflow Interrupt Enable
;
; OCIE1A - Timer/Counter 1 Vergleichszahl A Interrupt - 1: Int bei Erreichen Stand in A
;
}
  // Timer/Counter Interrupt Mask Register
  // Bit 4 – OCIE1A: Timer/Counter0 Overflow Interrupt Enable
  TIMSK:=%00010000;    // OCIE1A=1 -> Interrupt bei Output Compare A


end Init_TIMER1_COMPA;







procedure InitPWM1;
begin
  // Timer/Counter 1 Control Register B
//  TCCR1B:=%00000000;    // kei  Takt (Timer/Zaehler ist gestoppt)
  TCCR1B:=%00000001;    // kein Vorteiler
//  TCCR1B:=%00000010;    // Vorteiler = Takt/8
//  TCCR1B:=%00000011;    // Vorteiler = Takt/64
//  TCCR1B:=%00000100;    // Vorteiler = Takt/256
//  TCCR1B:=%00000101;    // Vorteiler = Takt/1024
//  TCCR1B:=%00000110;    // Externe Taktquelle fallende Flanke
//  TCCR1B:=%00000111;    // Externe Taktquelle steigende Flanke

  // Timer/Counter 1 Control Register A
  // Bit 7   – COM1A1 - Compare Output Mode for channel A
  // Bit 6   – COM1A0
  //           0 0 - kein PWM
  //           0 1 - kein PWM
  //           1 0 - PWM-Ausgang
  //           1 1 - PWM invertiert
  // Bit 5:4 – COM1B1 - Compare Output Mode for channel B
  //           0 0 - fuer Standard PWM (Komparator B)
  // Bit 3   - FOC1A - 0 fuer PWM Komp A
  // Bit 1:0 - WGM11, WGM10
  //           0 0 - kein PWM
  //           0 1 -  8bit PWM-Betrieb
  //           1 0 -  9bit PWM-Betrieb
  //           1 1 - 10bit PWM-Betrieb
  TCCR1A:=%10000001;

end InitPWM1;



// Timer / Counter 2



procedure InitCTC2(OC_Wert : Byte);
begin
  // Timer/Counter Control Register 0
//  TCCR2:=%00001000;    // kei  Takt (Timer/Zaehler ist gestoppt)
//  TCCR2:=%00001001;    // kein Vorteiler
//  TCCR2:=%00001010;    // Vorteiler = Takt/8
  TCCR2:=%00001011;    // Vorteiler = Takt/64
//  TCCR2:=%00001100;    // Vorteiler = Takt/256
//  TCCR2:=%00001101;    // Vorteiler = Takt/1024
//  TCCR2:=%00001110;    // Externe Taktquelle an T0 Pin. Takt an der fallenden Flanke
//  TCCR2:=%00001111;    // Externe Taktquelle an T0 Pin. Takt an der steigenden Flanke

  OCR2:=OC_Wert;

  // Timer/Counter Interrupt Mask Register
  // Bit 7 – OCIE2: Timer/Counter2 Output Compare Match Interrupt Enable
  TIMSK:=%10000000;    // OCIE2=1 -> Interrupt bei CTC
end InitCTC2;






procedure InitPWM2;
begin
  // Timer/Counter 2 Control Register
  // Bit 6,3 – WGM21:0: Waveform Generation Mode
  //    1 1  - FastPWM
  // Bit 5:4 – COM21:0: Compare Match Output Mode
  //    1 0  - Clear OC2 on Compare Match, set OC2 at BOTTOM, (non-inverting mode)
//  TCCR2:=%01101000;    // kei  Takt (Timer/Zaehler ist gestoppt)
  TCCR2:=%01101001;    // kein Vorteiler
//  TCCR2:=%01101010;    // Vorteiler = Takt/8
//  TCCR2:=%01101011;    // Vorteiler = Takt/64
//  TCCR2:=%01101100;    // Vorteiler = Takt/256
//  TCCR2:=%01101101;    // Vorteiler = Takt/1024
//  TCCR2:=%01101110;    // Externe Taktquelle fallende Flanke
//  TCCR2:=%01101111;    // Externe Taktquelle steigende Flanke

end InitPWM2;




