
// ******************************
// *** serielle Schnittstelle ***
// ******************************


VAR
  SER0_Zeichen : Byte;
  SER0_String : String[40];

  SER0_wait_before_sending : Boolean;

  xon_aktiv : Boolean;

  SER0_detected : Boolean;



procedure INIT_SER0_PORT;
begin

{
 Initialisieren der seriellen Schnittstelle
 ---------------------------------------------

; festlegen der Baudrate fuer die serielle Schnittstelle
}

    UBRR0H := 0;
    UBRR0L := UBRR_VAL;

{
Bit 4 RXEN: Receiver Enable
Bit 3 TXEN: Transmitter Enable
}
    UCSR0B := %00011000;

{
URSEL UMSEL = 01 = Synchronous USART
URSEL UMSEL = 00 = Asynchronous USART
UPM1 UPM0   = 00 = ParityMode Disable (no Parity)
USBS        =  0 = USART Stop Bit Select - 0 -> 1-bit
UCSZ1 UCSZ0 = 11 = Character Size Settings -> 8-bit
UCPOL       =  0 = Clock Polarity

URSEL UMSEL UPM1 UPM0 USBS UCSZ1 UCSZ0 UCPOL
    0     0    0    0    0     1     1     0
}
    UCSR0C := %00000110;


    SER0_wait_before_sending:=false;

    xon_aktiv:=true;

    SER0_detected:=false;


end INIT_SER0_PORT;





procedure RX0_Puffer_abfragen;
begin
  // wurde ein Byte durch die UART empfangen, so ist das RXC0-Bit gesetzt
  // ist RXC0-Bit = 0, dann ist der Empfangs-Puffer leer
  if BitHigh(RXC0) then
    SER0_Zeichen:=UDR0;
  else
    SER0_Zeichen:=0;
  endif; { if BitHigh(RXC0) then }
end RX0_Puffer_abfragen;








procedure RX0_wait_before_sending_abfragen;
begin

  RX0_Puffer_abfragen;

  // PC mit USB-Serial angeschlossen
  if SER0_Zeichen=xon then
    SER0_wait_before_sending:=false;
    xon_aktiv:=true;
    SER0_detected:=true;
  endif;

  // RPI angeschlossen
  if SER0_Zeichen=xoff then
    SER0_wait_before_sending:=true;
    xon_aktiv:=false;
    SER0_detected:=true;
  endif;


end RX0_wait_before_sending_abfragen;











procedure sende_Zeichen0(s_Zeichen0 : Byte);
begin

  if xon_aktiv then

    while SER0_wait_before_sending do
      RX0_Puffer_abfragen;
      if SER0_Zeichen=xon then
        SER0_wait_before_sending:=false;
      endif;
    endwhile;

  endif;


  // wenn UDRE0 gesetzt ist, so ist der Puffer leer
  // -> warten solange UDRE0 = 0
  while BitLow(UDRE0) do

  endwhile;

  UDR0:=s_Zeichen0;

  SER0_wait_before_sending:=true;

end sende_Zeichen0;




// SER0_String an SER0
procedure Write_SER0;
var
  c1 : Char;
  Laenge : Byte;
  b1 : Byte;
  b2 : Byte;
begin

  Laenge:=Length(SER0_String);
  for b1:=1 to Laenge do
    c1:=SER0_String[b1];
    b2:=CharToByte(c1);
    sende_Zeichen0(b2);
  endfor;

end Write_SER0;




procedure WriteLn_SER0;
begin
  Write_SER0;
  sende_Zeichen0($0D); // CR
  sende_Zeichen0($0A); // LF
end WriteLn_SER0;









