
// ****************************
// *** ITP3-Server-Routinen ***
// *** Softwareteil         ***
// ****************************


CONST

  // Befehlscode fuer ITP3
  ITP3_WHO_AM_I   : Byte = 01;
  ITP3_ERROR_CODE : Byte = 02;

  ITP3_CONOUT : Byte = 03;
  ITP3_CONST  : Byte = 04;
  ITP3_CONIN  : Byte = 05;





procedure ITP3_Init;
begin

{
A0 - DB0_ITP3 = E
A1 - DB1_ITP3 = E
A2 - DB2_ITP3 = E
A3 - DB3_ITP3 = E
A4 - DS0_ITP3 = A / H
A5 - DS1_ITP3 = A / H
A6 - DS2_ITP3 = A / H
A7 - DS3_ITP3 = A / H
}
  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

  // Device Select auf Ausgang
  SetBit(DS0_A);
  SetBit(DS0_D);

{
B0 - DR_ITP3  = E
}
  ClearBit(DR_D);  // DR auf Eingang


end ITP3_Init;







// Byte senden



procedure ITP3_sende_Halbbyte(SB : Byte);
var
  b1 : Byte;
begin

  // D0 setzen
  b1:=SB and %00000001;
  if b1=0 then
    ClearBit(D0_A);
  else
    SetBit(D0_A);
  endif;

  // D1 setzen
  b1:=SB and %00000010;
  if b1=0 then
    ClearBit(D1_A);
  else
    SetBit(D1_A);
  endif;

  // D2 setzen
  b1:=SB and %00000100;
  if b1=0 then
    ClearBit(D2_A);
  else
    SetBit(D2_A);
  endif;

  // D3 setzen
  b1:=SB and %00001000;
  if b1=0 then
    ClearBit(D3_A);
  else
    SetBit(D3_A);
  endif;

  // jetzt wird dem Client mitgeteilt, das Daten fuer ihn anliegen
  ClearBit(DS0_A);

  // warten solange der Client die Daten uebernimmt
  SetBit(LED_Rot);
  while BitHigh(DR_E) do
  endwhile;
  ClearBit(LED_Rot);

  // Client hat die Daten uebernommen,
  // die DS-Leitung wird wieder frei gegeben
  SetBit(DS0_A);

  // warten solange, bis der Client die DR-Leitung
  // wieder frei gibt
  SetBit(LED_Gelb);
  while BitLow(DR_E) do
  endwhile;
  ClearBit(LED_Gelb);

end ITP3_sende_Halbbyte;




procedure ITP3_sende_Byte(SB : Byte);
var
  b1 : Byte;
begin

  // alle Datenleitungen auf Ausgang schalten
  SetBit(D0_D);
  SetBit(D1_D);
  SetBit(D2_D);
  SetBit(D3_D);

  // Steuerleitungen setzen
  SetBit(DS0_A);    // DS auf 1
  SetBit(DS0_D);    // DS auf Ausgang
  ClearBit(DR_D);  // DR auf Eingang

  // senden des Low-Halbbyte
  b1:=SB;
  b1:=b1 and %00001111;
  ITP3_sende_Halbbyte(b1);

  // senden des High-Halbbyte
  b1:=SB;
  SHR(b1);
  SHR(b1);
  SHR(b1);
  SHR(b1);
  ITP3_sende_Halbbyte(b1);

  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

end ITP3_sende_Byte;





// Byte empfangen

function ITP3_empfange_Halbbyte : Byte;
var
  Result_Byte : Byte;
begin

  // SA

  // nun wird dem Client durch Aktivierung
  // der DSX-Leitung mitgeteilt,
  // dass der Server das naechste Halbbyte
  // empfangen kann
  ClearBit(DS0_A);

  // SB

  // warten, solange bis der Client das
  // Halbbyte bereit stellt
  SetBit(LED_Rot);
  while BitHigh(DR_E) do
  endwhile;
  ClearBit(LED_Rot);

  // SC

  // nun liegt das Halbbyte an

  Result_Byte:=%00000000;

  // D0 einlesen
  if BitHigh(D0_E) then
    Result_Byte:=Result_Byte or %00000001;
  endif;

  // D1 einlesen
  if BitHigh(D1_E) then
    Result_Byte:=Result_Byte or %00000010;
  endif;

  // D2 einlesen
  if BitHigh(D2_E) then
    Result_Byte:=Result_Byte or %00000100;
  endif;

  // D3 einlesen
  if BitHigh(D3_E) then
    Result_Byte:=Result_Byte or %00001000;
  endif;

  // SD

  // jetzt wird dem Client mitgeteilt,
  // das der Server das Halbbyte
  // uebernommen hat
  SetBit(DS0_A);

  // SE

  // warten solange bis der Client die
  // Bestaetigung angenommen hat
  SetBit(LED_Gelb);
  while BitLow(DR_E) do
  endwhile;
  ClearBit(LED_Gelb);

  // SF

  Result:=Result_Byte;

end ITP3_empfange_Halbbyte;






function ITP3_empfange_Byte : Byte;
var
  Result_Byte_01 : Byte;
  Result_Byte_02 : Byte;
begin

  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

  // unteres Halbbyte
  Result_Byte_01:=ITP3_empfange_Halbbyte;

  // oberes Halbbyte
  Result_Byte_02:=ITP3_empfange_Halbbyte;

  SHL(Result_Byte_02);
  SHL(Result_Byte_02);
  SHL(Result_Byte_02);
  SHL(Result_Byte_02);

  Result_Byte_01:=Result_Byte_01+Result_Byte_02;

  Result:=Result_Byte_01;

end ITP3_empfange_Byte;






procedure ITP3_sende_CONOUT(CO_Byte : Byte);
begin

  ITP3_sende_Byte(ITP3_CONOUT);
  ITP3_sende_Byte(CO_Byte);

end ITP3_sende_CONOUT;





function ITP3_empfange_CONST : Byte;
var
  RESULT_Byte : Byte;
begin

  ITP3_sende_Byte(ITP3_CONST);
  RESULT_Byte:=ITP3_empfange_Byte;

  Result:=RESULT_Byte;

end ITP3_empfange_CONST;






function ITP3_empfange_CONIN : Byte;
var
  RESULT_Byte : Byte;
begin

  ITP3_sende_Byte(ITP3_CONIN);
  RESULT_Byte:=ITP3_empfange_Byte;

  Result:=RESULT_Byte;

end ITP3_empfange_CONIN;








