
// ****************************
// *** ITP3-Client-Routinen ***
// *** Software-Teil        ***
// ****************************


CONST

  // Befehlscode fuer ITP3
  ITP3_WHO_AM_I    : Byte = 01;
  ITP3_ERROR_CODE  : Byte = 02;

  ITP3_CONOUT  : Byte = 03;
  ITP3_CONST   : Byte = 04;
  ITP3_CONIN   : Byte = 05;

  ITP3_STD     : Byte = 06;
  ITP3_RD_SEC  : Byte = 07;
  ITP3_WR_SEC  : Byte = 08;



procedure ITP3_Client_Init;
begin

  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

  // Device Select auf Eingang
  ClearBit(DS_D);

  // DR auf Eingang
  ClearBit(DR_D);


end ITP3_Client_Init;







// Byte einlesen


function ITP3_Halbbyte_einlesen : Byte;
var
  b1 : Byte;
begin

  // warten, solange bis der Server das
  // zweite Halbbyte sendet
  ClearBit(LED_Blau);
  while BitHigh(DS_E) do
  endwhile;
  SetBit(LED_Blau);

  b1:=%00000000;

  // D0 einlesen
  if BitHigh(D0_E) then
    b1:=b1 or %00000001;
  endif;

  // D1 einlesen
  if BitHigh(D1_E) then
    b1:=b1 or %00000010;
  endif;

  // D2 einlesen
  if BitHigh(D2_E) then
    b1:=b1 or %00000100;
  endif;

  // D3 einlesen
  if BitHigh(D3_E) then
    b1:=b1 or %00001000;
  endif;

  // jetzt wird dem Server mitgeteilt,
  // das die Daten uebernommen wurden
  ClearBit(DR_A);

  // warten solange bis der Server die
  // Bestaetigung angenommen hat
  ClearBit(LED_Blau);
  while BitLow(DS_E) do
  endwhile;
  SetBit(LED_Blau);

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);

  Result:=b1;

end ITP3_Halbbyte_einlesen;




function ITP3_Byte_einlesen : Byte;
var
  b1 : Byte;
  Result_Byte : Byte;
begin

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);
  // DR auf Ausgang - sendebereit -
  SetBit(DR_D);

  Result_Byte:=ITP3_Halbbyte_einlesen;


  b1:=ITP3_Halbbyte_einlesen;

  // DR wieder auf Eingang fuer die anderen Clients
  ClearBit(DR_D);

  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);

  Result_Byte:=Result_Byte+b1;

  Result:=Result_Byte;

end ITP3_Byte_einlesen;












// Halbbyte ausgeben

procedure ITP3_Halbbyte_ausgeben(H_Byte : Byte);
var
  b1 : Byte;
begin

  // CB

  b1:=H_Byte and %00000001;

  // D0 ausgeben
  if b1=0 then
    ClearBit(D0_A);
  else
    SetBit(D0_A);
  endif;


  b1:=H_Byte and %00000010;

  // D1 ausgeben
  if b1=0 then
    ClearBit(D1_A);
  else
    SetBit(D1_A);
  endif;


  b1:=H_Byte and %00000100;

  // D2 ausgeben
  if b1=0 then
    ClearBit(D2_A);
  else
    SetBit(D2_A);
  endif;


  b1:=H_Byte and %00001000;

  // D3 ausgeben
  if b1=0 then
    ClearBit(D3_A);
  else
    SetBit(D3_A);
  endif;


  // jetzt wird dem Server mitgeteilt,
  // das die Daten zur Uebernahme
  // anliegen
  ClearBit(DR_A);

  // CC

  // warten solange bis der Server die
  // Daten eingelesen hat und dies
  // bestaetigt hat
  ClearBit(LED_Blau);
  while BitLow(DS_E) do
  endwhile;
  SetBit(LED_Blau);

  // CE

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);

  // CF

end ITP3_Halbbyte_ausgeben;




procedure ITP3_Byte_ausgeben(H_Byte : Byte);
var
  b1 : Byte;

begin

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);
  // DR auf Ausgang - Uebernahme der Steuerleitung
  SetBit(DR_D);


  // CA

  ClearBit(LED_Blau);
  while BitHigh(DS_E) do
  endwhile;
  SetBit(LED_Blau);

  // CB

  // alle Datenleitungen auf Ausgang schalten
  SetBit(D0_D);
  SetBit(D1_D);
  SetBit(D2_D);
  SetBit(D3_D);

  b1:=H_Byte and %00001111;
  ITP3_Halbbyte_ausgeben(b1);

  SHR(H_Byte);
  SHR(H_Byte);
  SHR(H_Byte);
  SHR(H_Byte);

  // warten, solange bis der Server das
  // untere Halbbyte gelesen hat
  ClearBit(LED_Gelb);
  while BitHigh(DS_E) do
  endwhile;
  SetBit(LED_Gelb);

  ITP3_Halbbyte_ausgeben(H_Byte);

  // DR wieder auf Eingang fuer die anderen Clients
  ClearBit(DR_D);

  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

end ITP3_Byte_ausgeben;










procedure ITP3_PRG_WHO_AM_I;
var
  DATA_Byte : Byte;
begin

  // DRIVE_D
  DATA_Byte:=%00001000;
  ITP3_Byte_ausgeben(DATA_Byte);
  WHO_AM_I_Schleife:=false;

end ITP3_PRG_WHO_AM_I;









procedure ITP3_PRG_STD;
var
  DATA_Byte : Byte;
  TRACK_L   : DWord;
begin

  // SECTOR
  DATA_Byte:=ITP3_Byte_einlesen;
  Sektor_Nr:=ByteToDWord(DATA_Byte);

  // TRACK_L
  DATA_Byte:=ITP3_Byte_einlesen;
  TRACK_L:=ByteToDWord(DATA_Byte);

  // TRACK_H
  DATA_Byte:=ITP3_Byte_einlesen;
  Spur_Nr:=ByteToDWord(DATA_Byte);
  Spur_Nr:=Spur_Nr*256;
  Spur_Nr:=Spur_Nr+TRACK_L;

  // DISKNO wird derzeit ignoriert
  DATA_Byte:=ITP3_Byte_einlesen;

  Spur_und_Sektor_in_Datei_Zeiger;

end ITP3_PRG_STD;




procedure ITP3_PRG_RD_SEC;
var
  DATA_Byte : Byte;
begin

  // LED-Gruen ein
  ClearBit(LED_Gruen);
  // LED-Gelb aus
  SetBit(LED_Gelb);

  LED_Gruen_Schalter:=true;

  Lese_einen_Sektor;

  for Sektor_Array_Zeiger:=0 to 127 do

    DATA_Byte:=Sektor_Array[Sektor_Array_Zeiger];
    ITP3_Byte_ausgeben(DATA_Byte);

  endfor;

end ITP3_PRG_RD_SEC;





procedure ITP3_PRG_WR_SEC;
var
  DATA_Byte : Byte;
begin


  // LED-Rot ein
  ClearBit(LED_Rot);
  // LED-Gelb aus
  SetBit(LED_Gelb);

  LED_Rot_Schalter:=true;

  for Sektor_Array_Zeiger:=0 to 127 do

    DATA_Byte:=ITP3_Byte_einlesen;
    Sektor_Array[Sektor_Array_Zeiger]:=DATA_Byte;

  endfor;

  Schreibe_einen_Sektor;

  Write_Schalter:=true;

end ITP3_PRG_WR_SEC;












procedure ITP3_Server_Abfrage;
var
  CODE_Byte : Byte;
begin

  if BitLow(DS_E) then

    CODE_Byte:=ITP3_Byte_einlesen;

    case CODE_Byte of

      ITP3_WHO_AM_I : ITP3_PRG_WHO_AM_I;
      |

      ITP3_STD    : ITP3_PRG_STD;
      |

      ITP3_RD_SEC : ITP3_PRG_RD_SEC;
      |

      ITP3_WR_SEC : ITP3_PRG_WR_SEC;
      |

    endcase;

  //  Anzeige_des_Fehlercodes;

  endif;

end ITP3_Server_Abfrage;







