
// ****************************
// *** ITP3-Client-Routinen ***
// *** Software-Teil        ***
// ****************************


CONST

  // Befehlscode fuer ITP3
  ITP3_WHO_AM_I    : Byte = 01;
  ITP3_ERROR_CODE  : Byte = 02;

  ITP3_CONOUT : Byte = 03;
  ITP3_CONST  : Byte = 04;
  ITP3_CONIN  : Byte = 05;





procedure ITP3_Client_Init;
begin

  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

  // Device Select auf Eingang
  ClearBit(DS_D);

  // DR auf Eingang
  ClearBit(DR_D);



  // LED Blau und Rot aktivieren
  SetBit(LED_BL_D);
  SetBit(LED_RT_D);


  ClearBit(LED_BL);
  ClearBit(LED_RT);

end ITP3_Client_Init;







// Byte einlesen


function ITP3_Halbbyte_einlesen : Byte;
var
  b1 : Byte;
begin

  // warten, solange bis der Server das
  // zweite Halbbyte sendet
  SetBit(LED_BL);
  while BitHigh(DS_E) do
  endwhile;
  ClearBit(LED_BL);

  b1:=%00000000;

  // D0 einlesen
  if BitHigh(D0_E) then
    b1:=b1 or %00000001;
  endif;

  // D1 einlesen
  if BitHigh(D1_E) then
    b1:=b1 or %00000010;
  endif;

  // D2 einlesen
  if BitHigh(D2_E) then
    b1:=b1 or %00000100;
  endif;

  // D3 einlesen
  if BitHigh(D3_E) then
    b1:=b1 or %00001000;
  endif;

  // jetzt wird dem Server mitgeteilt,
  // das die Daten uebernommen wurden
  ClearBit(DR_A);

  // warten solange bis der Server die
  // Bestaetigung angenommen hat
  SetBit(LED_BL);
  while BitLow(DS_E) do
  endwhile;
  ClearBit(LED_BL);

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);

  Result:=b1;

end ITP3_Halbbyte_einlesen;




function ITP3_Byte_einlesen : Byte;
var
  b1 : Byte;
  Result_Byte : Byte;
begin

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);
  // DR auf Ausgang - sendebereit -
  SetBit(DR_D);

  Result_Byte:=ITP3_Halbbyte_einlesen;


  b1:=ITP3_Halbbyte_einlesen;

  // DR wieder auf Eingang fuer die anderen Clients
  ClearBit(DR_D);

  SHL(b1);
  SHL(b1);
  SHL(b1);
  SHL(b1);

  Result_Byte:=Result_Byte+b1;

  Result:=Result_Byte;

end ITP3_Byte_einlesen;












// Halbbyte ausgeben

procedure ITP3_Halbbyte_ausgeben(H_Byte : Byte);
var
  b1 : Byte;
begin

  // CB

  b1:=H_Byte and %00000001;

  // D0 ausgeben
  if b1=0 then
    ClearBit(D0_A);
  else
    SetBit(D0_A);
  endif;


  b1:=H_Byte and %00000010;

  // D1 ausgeben
  if b1=0 then
    ClearBit(D1_A);
  else
    SetBit(D1_A);
  endif;


  b1:=H_Byte and %00000100;

  // D2 ausgeben
  if b1=0 then
    ClearBit(D2_A);
  else
    SetBit(D2_A);
  endif;


  b1:=H_Byte and %00001000;

  // D3 ausgeben
  if b1=0 then
    ClearBit(D3_A);
  else
    SetBit(D3_A);
  endif;


  // jetzt wird dem Server mitgeteilt,
  // das die Daten zur Uebernahme
  // anliegen
  ClearBit(DR_A);

  // CC

  // warten solange bis der Server die
  // Daten eingelesen hat und dies
  // bestaetigt hat
  SetBit(LED_BL);
  while BitLow(DS_E) do
  endwhile;
  ClearBit(LED_BL);

  // CE

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);

  // CF

end ITP3_Halbbyte_ausgeben;




procedure ITP3_Byte_ausgeben(H_Byte : Byte);
var
  b1 : Byte;

begin

  // DR auf High (Grundeinstellung)
  SetBit(DR_A);
  // DR auf Ausgang - Uebernahme der Steuerleitung
  SetBit(DR_D);


  // CA

  SetBit(LED_BL);
  while BitHigh(DS_E) do
  endwhile;
  ClearBit(LED_BL);

  // CB

  // alle Datenleitungen auf Ausgang schalten
  SetBit(D0_D);
  SetBit(D1_D);
  SetBit(D2_D);
  SetBit(D3_D);

  b1:=H_Byte and %00001111;
  ITP3_Halbbyte_ausgeben(b1);

  SHR(H_Byte);
  SHR(H_Byte);
  SHR(H_Byte);
  SHR(H_Byte);

  // warten, solange bis der Server das
  // untere Halbbyte gelesen hat
  SetBit(LED_BL);
  while BitHigh(DS_E) do
  endwhile;
  ClearBit(LED_BL);

  ITP3_Halbbyte_ausgeben(H_Byte);

  // DR wieder auf Eingang fuer die anderen Clients
  ClearBit(DR_D);

  // alle Datenleitungen auf Eingang schalten
  ClearBit(D0_D);
  ClearBit(D1_D);
  ClearBit(D2_D);
  ClearBit(D3_D);

end ITP3_Byte_ausgeben;






procedure ITP3_PRG_CONOUT;
var
  DATA_Byte : Byte;
begin
  DATA_Byte:=ITP3_Byte_einlesen;

  L_Zeichen:=DATA_Byte;

  CONOUT;

  Dec(LED_ein);

end ITP3_PRG_CONOUT;






procedure ITP3_PRG_WHO_AM_I;
var
  DATA_Byte : Byte;
begin

  DATA_Byte:=%10000000;
  ITP3_Byte_ausgeben(DATA_Byte);

  WHO_AM_I_Schleife:=false;

end ITP3_PRG_WHO_AM_I;






procedure ITP3_Server_Abfrage;
var
  CODE_Byte : Byte;
begin

  if BitLow(DS_E) then

    CODE_Byte:=ITP3_Byte_einlesen;

    case CODE_Byte of

      ITP3_WHO_AM_I : ITP3_PRG_WHO_AM_I;
      |

      ITP3_CONOUT : ITP3_PRG_CONOUT;
      |

    endcase;

  endif;

end ITP3_Server_Abfrage;






